// This script is part of the CHANGE MANAGEMENT app available in the App Store
// last modification date: Mar 19, 2020

switch(customTransID) {
    
    case 'START':
        reset();
        saveDate("START_DATE");
        item.STEP = 1;
        break;
    
    case 'CANCEL':
        item.STEP = 0;
        item.workflowItems.clear();
        item.deleteItem();
        break;
        
    case 'INITIATE_TASKS':
        initiateChangeTasks("CHANGE_ORDER");
        saveDate("CHANGE_START_DATE");
        item.STEP = 2;
        if(item.FAST_TRACK !== "No") item.FAST_TRACK = "No";
        break;
        
    case 'UPDATE_TASKS':
    case 'INITIATE_ADDITIONAL_TASKS':
        initiateChangeTasks("CHANGE_ORDER");
        break;
  
    case 'SUBMIT_TO_CCB_FAST_TRACK':
    case 'SUBMIT_TO_CCB':
        setRemainingApprovers("REQUIRED_APPROVERS", "ADDITIONAL_APPROVERS", "REMAINING_APPROVERS");
        updateEffortTotals();
        saveDate("CCB_START_DATE");
        setKPIRoundCount();
        item.STEP = 3;
        if(customTransID === "SUBMIT_TO_CCB") {
            item.CHANGE_COMPLETION_DATE = new Date();
        } else {
            if(item.FAST_TRACK === null) item.FAST_TRACK = "Yes";
        }
        break;
        
    case 'RETURN_REJECT':
        item.APPROVED_BY = [];
        item.REMAINING_APPROVERS = [];
        item.STEP = 1;
        break;
    
    case 'APPROVE':
        saveApproval("APPROVED_BY", "REMAINING_APPROVERS");
        proceedUponFinalApproval();
        break;
    
/*    case 'FINAL_APPROVAL_EXT_REVIEW':
        setRemainingConfirmations();
        transitionPreceedingProcesses();
        item.CCB_COMPLETION_DATE = new Date();
        item.EXTERNAL_REVIEW_START_DATE = new Date();
        break;*/
        
    case 'EXTERNAL_REJECTION_RECEIVED':
        item.APPROVED_BY = [];
        item.REMAINING_APPROVERS = [];
        item.STEP = 1;
        break;
        
    case 'EXTERNAL_APPROVAL_RECEIVED':
        proceedUponExternalApproval();
        break;
    
    case 'CONFIRM':
        saveApproval("CONFIRMED_BY", "REMAINING_CONFIRMATIONS");
        proceedUponFinalConfirmation();
        break;

}


// reset relevant data when cloning
function reset() {
    
    item.grid.clear();
    
    resetField("PROBLEM_REPORT");
    resetField("CHANGE_REQUEST");
    
    resetField("PENDING_TASKS_COUNT");
    resetField("TOTAL_TASKS_COUNT");
    
    item.REQUIRED_APPROVERS = [];
    item.APPROVED_BY = [];
    item.REMAINING_APPROVERS = [];   
    
    item.REQUIRED_CONFIRMATIONS = [];
    item.CONFIRMED_BY = [];
    item.REMAINING_CONFIRMATIONS = [];

    resetField("CHANGE_START_DATE");
    resetField("CHANGE_COMPLETION_DATE"); 
    resetField("CCB_START_DATE");
    resetField("CCB_COMPLETION_DATE");
    resetField("EXTERNAL_REVIEW_START_DATE");
    resetField("EXTERNAL_REVIEW_COMPLETION_DATE");
    resetField("IMPLEMENTATION_START_DATE");
    resetField("IMPLEMENTATION_COMPLETION_DATE");
    resetField("START_DATE");
    resetField("COMPLETION_DATE");
    
    resetField("TOTAL_EFFORT_PLANNED");
    resetField("TOTAL_EFFORT_ACTUAL");
    
    resetField("FAST_TRACK");
    resetField("CCB_REVIEW_ROUNDS");
    
}


// calculate total planned and actual efforts based on related tasks
function updateEffortTotals() {
    
    var effortPlanned = 0;
    var effortActual  = 0;
    
    for(var i = 0; i < item.project.children.length; i++) {
        
        var task    = item.project.children[i].target;
        var planned = (task.TARGET_EFFORT === null) ? 0 : task.TARGET_EFFORT;
        var actual  = (task.ACTUAL_EFFORT === null) ? 0 : task.ACTUAL_EFFORT;
        
        effortPlanned += planned;
        effortActual += actual;
        
    }
    
    if(item.TOTAL_EFFORT_PLANNED !== effortPlanned) item.TOTAL_EFFORT_PLANNED = effortPlanned;
    if(item.TOTAL_EFFORT_ACTUAL  !== effortActual ) item.TOTAL_EFFORT_ACTUAL  = effortActual;
    
}


// proceed upon final approval
function proceedUponFinalApproval() {
    
    if(item.REMAINING_APPROVERS.length > 1) return;
    
    item.CCB_COMPLETION_DATE = new Date();
    
    if(item.CUSTOMER_APPROVAL_REQUIRED === "Yes") {
        item.performWorkflowTransitionByCustomID("FINAL_APPROVAL_EXT_REVIEW", "CCB approval done");
        saveDate("EXTERNAL_REVIEW_START_DATE");
    } else {
        setRemainingApprovers("REQUIRED_CONFIRMATIONS", "ADDITIONAL_CONFIRMATIONS", "REMAINING_CONFIRMATIONS");
        item.performWorkflowTransitionByCustomID("FINAL_APPROVAL", "CCB approval done");
        if((item.REQUIRED_CONFIRMATIONS.length === 0) && (item.ADDITIONAL_CONFIRMATIONS.length === 0)) {
                item.performWorkflowTransitionByCustomID("FINAL_CONFIRMATION", "CCB approval done");
                item.STEP = 5;
                saveDate("COMPLETION_DATE");
        } else {
            item.IMPLEMENTATION_START_DATE = new Date();
            item.STEP = 4;
        }
        transitionPreceedingProcesses();
    }
    
}


// proceed upon external approval
function proceedUponExternalApproval() {
    
    transitionPreceedingProcesses();
    saveDate("EXTERNAL_REVIEW_COMPLETION_DATE");

    if((item.REQUIRED_CONFIRMATIONS.length > 0) || (item.ADDITIONAL_CONFIRMATIONS.length > 0)) {
        saveDate("IMPLEMENTATION_START_DATE");
        setRemainingApprovers("REQUIRED_CONFIRMATIONS", "ADDITIONAL_CONFIRMATIONS", "REMAINING_CONFIRMATIONS");
        item.STEP = 4;
    } else {
        item.STEP = 5;
        saveDate("COMPLETION_DATE");
        item.performWorkflowTransitionByCustomID("FINAL_CONFIRMATION", "No confirmations required");
    }
    
    
}

// proceed upon final confirmation
function proceedUponFinalConfirmation() {
   
    if(item.REMAINING_CONFIRMATIONS.length > 1) return;
    
    item.STEP = 5;
    
    saveDate("IMPLEMENTATION_COMPLETION_DATE");
    saveDate("COMPLETION_DATE");
    
    item.performWorkflowTransitionByCustomID("FINAL_CONFIRMATION", "All confirmations done");
    
}

function transitionPreceedingProcesses() {
    
    for(var i = 0; i < item.CHANGE_REQUESTS.length; i++) {
        
        var elemCR = item.CHANGE_REQUESTS[i];
        
        if(elemCR.descriptor.workflowState === "Change Order in progress") {
            elemCR.STEP = 5;
            elemCR.performWorkflowTransitionByCustomID("CO_COMPLETED", "Change Order completed");
            
            if(elemCR.PROBLEM_REPORT !== null) {
                
                var elemPR = elemCR.PROBLEM_REPORT;
                
                if(elemPR.descriptor.workflowState === "Change Request in progress") {
                    elemPR.performWorkflowTransitionByCustomID("CR_COMPLETED", "Change Request completed");
                }
                
            }
            
        }
        
    }
}