// This script is part of the CHANGE MANAGEMENT app available in  the App Store
// last modification date: June 27, 2023

switch(customTransID) {
    
    case 'START':
        var modified = false;
        if(item.TARGET_START_DATE === null) { item.TARGET_START_DATE = new Date(); modified = true; }
        if(item.TARGET_EFFORT     === null) { item.TARGET_EFFORT     = 0;          modified = true; }
        if(!modified) setMilestones();
        setAffectedItems();
        break;
    
    case 'ACKNOWLEDGE':
        saveDate("ACTUAL_START_DATE");
        break;
    
    case 'COMPLETE_TASK':
    case 'APPROVE_FINISH_TASK':
        saveDate("ACTUAL_COMPLETION_DATE");
        if(customTransID === "APPROVE_FINISH_TASK") saveDate("ACTUAL_REVIEW_DATE");
        calcEffort();
        updateParent();
        updateParentAffectedItems();
        break;
    
    case 'REQUEST_REWORK':
        saveDate("ACTUAL_REVIEW_DATE");
        break;
    
}

appendToThread("COMMENTS_LOG", true);


// set milestones based on target dates
function setMilestones() {
    
    setMilestone("In Work",   "ENTER", item.TARGET_START_DATE, null, 0, 10, true);
    setMilestone("Completed", "ENTER", item.TARGET_COMPLETION_DATE, null, 0, 100, true);
    
}


// add selected items to managed items tab
function setAffectedItems() {

    if(item.CHANGE_ORDER === null) return;

    for(var i = 0; i < item.CHANGE_ORDER.grid.length; i++) {

        var row = item.CHANGE_ORDER.grid[i];

        if(row.TASK === item) {
            for(var x = 0; x <  row.ITEMS_TO_CHANGE.length; x++) {
                item.DESCRIPTON = '5' + row.ITEMS_TO_CHANGE.length;
                item.workflowItems.addItem({RELATED:row.ITEMS_TO_CHANGE[x]})
            }
        }

    }

}


// calculate total effort based on grid details
function calcEffort() {
    
    var effort = 0;
    
    for(var i = 0; i < item.grid.length; i++) {
        var add = item.grid[i].EFFORT;
        if(add === null) add = 0;
        effort += add;
    }
    
    if(item.ACTUAL_EFFORT !== effort) item.ACTUAL_EFFORT = effort;
    
}

// Upon task completion transition parent process and remove entry from parent's grid
function updateParent() {
    
    var parent = item.CHANGE_ORDER;
    if(parent === null) parent = item.CHANGE_REQUEST;
    if(parent === null) return;

    var countPending    = 0;
    var effort          = 0;

    if(parent !== null)  {

        for(iGrid = 0; iGrid < parent.grid.length; iGrid++) {
            if(parent.grid[iGrid].TASK === item) {
                parent.grid[iGrid].remove();
                break;
            }
        }

        var children = parent.project.children;

        for(var i = children.length - 1; i >= 0; i--) {
            var add = children[i].target.ACTUAL_EFFORT;
            if(add !== null) effort += add;
            if(children[i].target.master.dmsID !== dmsID) {
               if(children[i].progress !== 100) {
                   countPending++;
               }
            }
        }

        if(countPending === 0) parent.performWorkflowTransitionByCustomID("TASKS_COMPLETED", "All tasks completed");
        parent.PENDING_TASKS_COUNT = countPending;
        
        if(parent.TOTAL_EFFORT_ACTUAL !== effort) parent.TOTAL_EFFORT_ACTUAL = effort;

    }
    
}
function updateParentAffectedItems() {

    var elemCO = item.CHANGE_ORDER;

    if(elemCO === null) return;

    for(var i = item.workflowItems.length - 1; i >= 0; i--) {

        var isConnected = false;

        for(var x = 0; x < elemCO.workflowItems.length; x++) {
            if(elemCO.workflowItems[x].id === item.workflowItems[i].id) {
                isConnected = true;
                continue;
            }
        }

        if(!isConnected) elemCO.workflowItems.addItem({RELATED: item.workflowItems[i].item});

    }

}