// This script is part of the app DESIGN REVIEWS available in the App Store
// last modification date: Nov 26, 2019

var wsDesignReviews     = "WS_DESIGN_REVIEWS";
var wsDesignReviewTasks = "WS_DESIGN_REVIEW_TASKS";

switch(customTransID) {
    
    case 'INITIATE':
        setNumber();
        reset();
        break;
    
    case 'BEGIN_REVIEW':
        item.ACTUAL_REVIEW_DATE = new Date();
        break;
    
    case 'RETURN':
        item.ACTUAL_REVIEW_DATE = null;
        break;
        
    case 'CLOSE_REVIEW':
        setApprovers();
        break;
        
    case 'RETURN_2':
        item.PENDING_APPROVERS = [];
        break;
    
    case 'APPROVE':
        updateApprovers();
        break;
        
    case 'FINISH_REVIEW_NO_REWORK':
    case 'ALL_TASKS_COMPLETED':
        item.COMPLETION_DATE = new Date();
        createFollowup();
        break;
    
}


// set Sequence & ID
function setNumber() {
        
    if(item.ISFOLLOWUP === true) {
        item.ISFOLLOWUP = false;
        setOwner();
    } else {
    	var seq = new Sequencer(wsDesignReviews);
        item.ID = "DR-" + ('000000' + seq.nextValue().toString()).slice(-6);
        item.SEQUENCE = 1;
    }
    
}
function setOwner() {
    
    var prev = item.PRECEDING_DESIGN_REVIEW;
    
    if(prev !== null) {
        if(prev.descriptor.ownerID !== userID) {
            item.master.owner = Security.loadUser(prev.descriptor.ownerID);
        }
    }
    
}


// reset item details when creating new records
function reset() {
    
    resetField("FOLLOWUP_REVIEW_REQUIRED");
    resetField("FOLLOWOUP_REVIEW_DATE");
    resetField("COMMENTS");
    
    resetField("APPROVAL_DATE");
    
    resetField("ACTUAL_START_DATE");
    resetField("COMPLETION_DATE");
    resetField("START_DATE");
    resetField("DERIVED_TASKS_COUNT");
    resetField("PENDING_TASKS_COUNT");
    resetField("REWORK_REQUIRED");

    if(item.PENDING_APPROVERS.length > 0) item.PENDING_APPROVERS = [];
    
    item.grid.clear();
    
}
function resetField(fieldName) {
    if(item[fieldName] !== null) item[fieldName] = null;
}


// set list of approvers in "approvers pending"field
function setApprovers() {
    
    if(item.REQUIRED_APPROVERS.length === 0) {
        autoTransition();
    } else {
        for(var i = 0; i < item.REQUIRED_APPROVERS.length; i++) {
            item.PENDING_APPROVERS.add(item.REQUIRED_APPROVERS[i]);
        }
    }
    
}


// upon approval, store information
function updateApprovers() {
    
    var user = Security.loadUser(userID);
    var userName = user.lastName + ", " + user.firstName;
    
    item.PENDING_APPROVERS.remove(userName);
    
    if(item.PENDING_APPROVERS.length === 1) autoTransition();
    
}


// transition to next status once all approvals are done
function autoTransition() {
    
    sendReport();
    
    item.APPROVAL_DATE = new Date();
    item.DERIVED_TASKS_COUNT = item.grid.length;
    item.PENDING_TASKS_COUNT = item.grid.length;
    
    if(item.grid.length === 0) {
        item.COMPLETION_DATE = new Date();
        item.REWORK_REQUIRED = "No";
        createFollowup();
        item.performWorkflowTransitionByCustomID("FINISH_REVIEW_NO_REWORK", "All signatures done, Design Review process completed without rework");
    } else {
        item.REWORK_REQUIRED = "Yes";
        item.performWorkflowTransitionByCustomID("FINISH_REVIEW_AND_START_REWORK", "All signatures done, starting tasks");
        createTasks();
        
    }
}
function sendReport() {
    
    var ownerName   = item.master.owner.lastName + ", " + item.master.owner.firstName;
    var to          = item.master.owner.email;
    var coordinator = item.COORDINATOR;
    var addressees  = [];
    
    if(ownerName !== coordinator) addressees.push(item.COORDINATOR);
    
    for(var i = 0; i < item.REQUIRED_APPROVERS.length; i++) {
        addressees.push(item.REQUIRED_APPROVERS[i]);
    }
    for(var j = 0; j < item.DISTRIBUTION_LIST.length; j++) {
        var nameDist = item.DISTRIBUTION_LIST[j];
        if(addressees.indexOf(nameDist) === -1) addressees.push(nameDist);
    }

    for(var k = 0;  k < addressees.length; k++) {
        
        var addressee   = addressees[k];
        var name        = addressee.split(", ");
        var userInfo    = Security.searchUsers({lastName:name[0], firstName:name[1]}); 
        var user        = Security.loadUser(userInfo[0].id);
        
        to += "," + user.email;
        
    }
    
    var body = getPrintView("Summary");
        body = body.replace(/___OWNER___/g, item.master.owner.email);
    
	var email = new Email();
	    email.to = to;
		email.subject = "Summary report of Design Review " + item.descriptor.descriptor;
		email.body = body;
        email.send();
    
}

function createTasks() {
    
    for(var i = 0; i < item.grid.length; i++) {
        
        var row = item.grid[i];
        var elemTask = createItem(wsDesignReviewTasks);
            elemTask.DESIGN_REVIEW = item;
            elemTask.TITLE = row.TITLE;
            elemTask.DESCRIPTION = row.DESCRIPTION;
            elemTask.TARGET_START_DATE = row.TARGET_START_DATE;
            elemTask.TARGET_COMPLETION_DATE = row.TARGET_COMPLETION_DATE;
            elemTask.ASSIGNEE = row.ASSIGNEE;
            elemTask.REVIEWER = row.REVIEWER;
            
        row.LINK = elemTask;
        item.project.addTask(elemTask);
        
    }
    
}

function createFollowup() {
    
    if(item.FOLLOWUP_REVIEW === "No") return;
    
    var sequence = item.SEQUENCE + 1;
    
    var elemFollowup = createItem(wsDesignReviews);
        elemFollowup.ID = item.ID;
        elemFollowup.SEQUENCE = sequence;
        elemFollowup.TYPE = item.TYPE;
        elemFollowup.TITLE = item.TITLE;
        elemFollowup.COORDINATOR = item.COORDINATOR;
        elemFollowup.TARGET_REVIEW_DATE = item.FOLLOWUP_REVIEW_DATE;
        elemFollowup.PROJECT = item.PROJECT;
        elemFollowup.PRECEDING_DESIGN_REVIEW = item;
        elemFollowup.ISFOLLOWUP = true;
        
    item.FOLLOWUP_DESIGN_REVIEW = elemFollowup;
    
}
