// This script is part of the IDEA MANAGEMENT app available in the App Store
// Last modification date: Apr 27, 2023


switch(customTransID) {
    
    case 'START':
        reset();
        setNumber();
        break;

    case 'SUBMIT':
    case 'RETURN_TO_INITIAL_REVIEW':
        item.STEP = 2;
        break;

    case 'RETURN__REQUEST_DETAILS':
    case 'RETURN':
        item.STEP = 1;
        break;

    case 'SUBMIT_FOR_ANALYSIS':
    case 'END_VOTING':
        item.STEP = 3;
        break;

    case 'INITIATE_VOTING':
        item.STEP = 4;
        setPendingVotes();
        break;

    case 'SHELVE':
        item.STEP = 88;
        saveDecisionDetails('Shelved');
        break;

    case 'APPROVE__SELECT':
        item.STEP = 5;
        saveDecisionDetails('Selected');
        break;

    case 'REJECT':
        item.STEP = 77;
        saveDecisionDetails('Rejected');
        break;

    case 'CONTINUE_1':
    case 'CONTINUE_2':
        item.STEP = 3;    
        resetDecisionDetails();
        break;        

    case 'COMPLETE':
        item.STEP = 6;
        item.COMPLETION_DATE = new Date();
        break;

    default:
        if(customTransID.indexOf('VOTE_') === 0) {
            var vote = customTransID.split('VOTE_')[1];
            saveVote(vote);
        } else if(customTransID.indexOf('CANCEL_') === 0) {
            item.STEP = 99;
        }
    
}


// START
function reset() {

    item.STEP           = 1;
    item.CREATION_DATE  = new Date();
    item.CREATED_BY     = getUserName();

    item.TOTAL_SCORE        = null;
    item.SCORE_LABEL        = null;

    item.TOTAL_VOTES        = null;
    item.PENDING_VOTES      = [];
    item.VOTES_LEFT         = null;
    item.LIKES              = null;
    item.NEUTRALS           = null;
    item.DISLIKES           = null;
    item.LIKES_RATIO        = null;
    
    item.APPROVED_BY        = null;
    item.APPROVAL_DATE      = null;
    item.APPROVAL_COMMENTS  = null;
    
    item.VOTING_CONDUCTED   = null;
    item.VOTING_ITERATIONS  = 0;
    item.DECISION           = null;
    item.COMPLETION_DATE    = null;

    item.RELATED_PRODUCT_LINE           = null;
    item.PREVIOUS_RELATED_PRODUCT_LINE  = null;
    
    item.RELATED_PRODUCTS                   = [];
    item.PREVIOUS_RELATED_PRODUCTS          = [];
    item.RELATED_REQUIREMENTS               = [];
    item.PREVIOUS_RELATED_REQUIREMENTS      = [];
    item.RELATED_CHANGE_REQUESTS            = [];
    item.PREVIOUS_RELATED_CHANGE_REQUESTS   = [];
    
    item.grid.clear();
    
}
function setNumber() {
    
    var seq = new Sequencer('Ideas');
    item.ID = prefix + ('000000' + seq.nextValue().toString()).slice(-6);
    
}


// VOTING
function setPendingVotes() {

    item.VOTING_CONDUCTED   = 'Yes';
    item.VOTING_ITERATIONS  = item.VOTING_ITERATIONS + 1;
    item.TOTAL_VOTES        = 0;
    item.LIKES              = 0;
    item.NEUTRALS           = 0;
    item.DISLIKES           = 0;
    item.LIKES_RATIO        = 0;

    var users = Security.listUsersInGroup(groupVoting);
		    
    for(var i = 0; i < users.length; i++) {

        var user = Security.loadUser(users[i]);
        var userName = user.lastName + ", " + user.firstName
        
        item['PENDING_VOTES'].add(userName);

    }

    if(item.VOTING_USERS !== null) {
        for(var j = 0; j < item.VOTING_USERS.length; j++) {
            item['PENDING_VOTES'].add(item.VOTING_USERS[j]);
        }
    }


    item.VOTES_LEFT = item['PENDING_VOTES'].length;

}
function saveVote(vote) {

    var user     = Security.loadUser(userID);
    var userName = user.lastName + ', ' + user.firstName;
    var row      = item.grid.addRow();

    row.USERNAME  = userName;
    row.DATE      = new Date();
    row.VOTE      = vote;
    row.ITERATION = item.VOTING_ITERATIONS;
    row.COMMENTS  = item.workflowActions[0].comments;

    switch(vote.toLowerCase()) {

        case 'like'     : item.LIKES    = item.LIKES    + 1; break;
        case 'neutral'  : item.NEUTRALS = item.NEUTRALS + 1; break;
        case 'dislike'  : item.DISLIKES = item.DISLIKES + 1; break;

    }

    var totalVotes  = item.LIKES + item.NEUTRAL + item.DISLIKES;
    var ratio       = 0;
    
    if(item.LIKES > 0) ratio = Math.round(item.LIKES * 100 / totalVotes, 0);

    item.TOTAL_VOTES    = totalVotes;
    item.VOTES_LEFT     = item['PENDING_VOTES'].length - 1;
    item.LIKES_RATIO    = ratio;
    
    item.PENDING_VOTES.remove(userName);
    

    if(item.PENDING_VOTES.length === 1) {
        item.performWorkflowTransitionByCustomID('END_VOTING', 'All requested votes received, closing idea voting automatically');
        item.STEP = 3;
    }

}



// DECISION
function saveDecisionDetails(decision) {

    item.DECISION           = decision;
    item.DECISION_DATE      = new Date();
    item.DECISION_BY        = getUserName();
    item.DECISION_COMMENTS  = item.workflowActions[0].comments;

    if(item.VOTING_CONDUCTED === null) item.VOTING_CONDUCTED = 'No';

}
function resetDecisionDetails() {

    item.DECISION           = '';
    item.DECISION_DATE      = null;
    item.DECISION_BY        = null;
    item.DECISION_COMMENTS  = null;

}


// Utilities
function getUserName() {
    var usr = Security.loadUser(userID);
    return usr.lastName + ', ' + usr.firstName;
}