// This script is part of the NEW PRODUCT INTRODUCTION app available in the App Store
// last modification date: Jan 18, 2021

setMailAddress();
updateRelatedProject("TEAM", "PREVIOUS_TEAM");
updateRelatedGrid("TEAM", "PREVIOUS_TEAM", "MEMBER_SINCE");


// Set default E-Mail address to match user account
function setMailAddress() {
    
    if(item.USER_ACCOUNT === null) return;
    
    var userAccount = item.USER_ACCOUNT.split(", ");
    var userInfo    = Security.searchUsers({lastName:userAccount[0], firstName:userAccount[1]}); 
    var userData    = Security.loadUser(userInfo[0].id);
		        
    if(item.EMAIL !== userData.email) item.EMAIL = userData.email;
    
}


/* ----------------------------------------------------------------------------------------------------
    Add current object with details to selected parent's grid
    - fieldNew   : Pick list field that defines the parent whose grid needs to be updated
    - fieldOld   : Pick list field to store that last selected parent (to recognize changes)
    - columnDate : Column of grid to store date of addition to grid (optional)
    ---------------------------------------------------------------------------------------------------- */
function updateRelatedGrid(fieldNew, fieldOld, columnDate) {
    
    var valueNew = item[fieldNew];
    var valueOld = item[fieldOld];
    
    if(valueNew !== valueOld) {
    
        addToGrid(valueNew, columnDate);
        removeFromGrid(valueOld);
        item[fieldOld] = item[fieldNew];
    
    } else {
        
        updateExistingGrid(valueNew);
        
    }

}
function updateExistingGrid(parent) {
    
    if(parent !== null) {
    
        var bAdd    = true;
        var grid    = parent.grid;
        
        for(var i = 0; i < grid.length; i++) {
            var row = grid[i];
            if(row.ITEM == item) {
                bAdd = false;
                syncGridRow(row, item);
            }
        }
        
        if(bAdd) addToGrid(parent);
    
    }
    
}
function addToGrid(parent, columnDate) {

    if(parent !== null) {

        var grid = parent.grid;
        var row  = grid.addRow();
        
        syncGridRow(row, item);
        
        if(columnDate !== null) row[columnDate] = new Date();
        
    }
    
}
function syncGridRow(row, item) {
    
    var columns = Object.keys(row);
    var fields  = Object.keys(item);
    
    for(var i = 0; i < columns.length; i++) {
        
        var column = columns[i];
    
        if(column == "ITEM") { row.ITEM = item; }
        else if(column == "WORKFLOW_STATUS") { row.WORKFLOW_STATUS = item.descriptor.workflowState; }
        else {
            
            for(var j = 0; j < fields.length; j++) {
        
                var field = fields[j];
                if(field == column)  row[column] = item[field];
                
            }
            
        }
    
    }
    
}
function removeFromGrid(parent) {
    
   if(parent !== null) {
    
        var grid = parent.grid;
        
        for(var i = grid.length - 1; i >= 0; i--) {
            
            var row = grid[i];
            if(row.ITEM == item) row.remove();
            
        }
    
   }
    
}


/* ----------------------------------------------------------------------------------------------------
    Add current item to project management tab of item selected in pick list field
    - fieldNew  : Pick list field in which users can select the parent (where the current item will be added)
    - fieldOld  : Pick list field to store previously selected parent (will be set by script automatically)
   ---------------------------------------------------------------------------------------------------- */
function updateRelatedProject(fieldNew, fieldOld) {
    
    var elemNew = item[fieldNew];
    var elemOld = item[fieldOld];
  
    if(elemNew !== elemOld) {

        if(elemNew !== null) elemNew.project.addTask(item, new Date(), new Date()); 
        
        if(elemOld !== null) {
        
            var elemTasks = elemOld.project.children;
            var itemId = item.master.dmsID;
            
            for(var i = elemTasks.length - 1; i >= 0; i--) {
                
                var elemTask = elemTasks[i];
                
                if(elemTask !== null) {
                    
                    if(elemTask.item !== null) {
                
                        var elemTaskId = elemTask.item.master.dmsID;
                        
                        if(elemTaskId == itemId) {
                            elemTask.remove(); 
                            break;
                        }
                        
                    }
                    
                }
                
            }
        
        }
        
    }

}