// This script is part of the CHANGE MANAGEMENT app available in the App Store
// Last Modification Date : June 14, 2023

switch(customTransID) {
    
    case 'START':
        reset();
        setStatusInformation(10, "Creation");
        break;
        
    case 'CLOSE_1':
    case 'CLOSE_2':
        setStatusInformation(100, "Closed");
        saveDate("CLOSURE_DATE");
        item.CAPA_REQUESTED = "No";
        item.CR_REQUESTED = "No"; 
        if(customTransID === "CLOSE_2") saveReviewer();
        break;

    case 'RETURN':
        setStatusInformation(10, "Creation");
        clearMilestones();
        break;  

    case 'SUBMIT':
        setStatusInformation(30, "Review");
        setMilestone("Review", "EXIT", null, 3, 0, 0, true);
        saveDate("SUBMITTAL_DATE");
        setAffectedItems();
        break;
        
    case 'COMPLETE_1':
        setStatusInformation(100, "Complete");
        saveDate("REVIEW_DATE");
        saveDate("COMPLETION_DATE");
        saveReviewer();
        item.CAPA_REQUESTED = "No";
        item.CR_REQUESTED = "No";  
        break;
        
    case 'SUBMIT_TO_TECHNICIAN':
        setStatusInformation(60, "Analysis");
        setMilestone("Technical Analysis", "EXIT", item.TECHNICAL_ANALYSIS_TARGET_DATE, null, 0, 80, true);
        saveDate("REVIEW_DATE");
        saveReviewer();
        break;
        
    case 'RETURN_TO_REVIEW':
        setStatusInformation(40, "Review");
        removeMilestones("Technical Analysis");
        break;

    case 'COMPLETE_2':
        setStatusInformation(100, "Complete");
        saveDate("COMPLETION_DATE");
        saveTechnicalAnalysisDetails();
        item.CAPA_REQUESTED = "No";
        item.CR_REQUESTED = "No";   
        break;
    
    case 'INITIATE_CAPA':
        setStatusInformation(80, "Processing Actions");
        saveTechnicalAnalysisDetails();
        item.CAPA_REQUESTED = "Yes";
        item.CR_REQUESTED = "No";
        initiateCAPA();
        break;
        
    case 'INITIATE_CHANGE_REQUEST':
        setStatusInformation(80, "Processing Actions");
        saveTechnicalAnalysisDetails();
        item.CAPA_REQUESTED = "No";
        item.CR_REQUESTED = "Yes";
        item.CR_CREATION_DATE = new Date();
        createChangeRequest();
        break;
        
    case 'INITIATE_CHANGE_REQUEST_FROM_CAPA':
        item.CR_REQUESTED = "Yes";
        createChangeRequest();
        break;
        
    case 'CAPA_COMPLETED':
    case 'CR_COMPLETED':
        setStatusInformation(100, "Completed");
        saveDate("COMPLETION_DATE");
        break;
    
}

// reset relevant fields upon creation (/cloning)
function reset() {
    
    resetField("REVIEWER");
    resetField("PROBLEM_VERIFICATION");
    resetField("ROOT_CAUSE_CODE");
    resetField("EXPECTED_COST");
    resetField("AFFECTED_SUPPLIER");
    resetField("ACTION_TAKEN");
    resetField("COMMMENTS");
    resetField("REVIEWED_BY");
    resetField("REVIEW_DATE");
    
    resetField("TECHNICIAN");
    resetField("TECHNICAL_ANALYSIS_TARGET_DATE");
    resetField("TECHNICAL_ANALYSIS_COMPLETION_DATE");
    resetField("TECHNICAL_ANALYSIS_COMMENTS");
    
    resetField("CAPA_REQUESTED");
    resetField("CAPA");
    resetField("CAPA_CREATION_DATE");
    resetField("CR_REQUESTED");
    resetField("CHANGE_REQUEST");
    resetField("CR_CREATION_DATE");
    
    resetField("SUBMITTAL_DATE");
    resetField("COMPLETION_DATE");
    resetField("CLOSURE_DATE");
    
}


// set status label and percent complete information
function setStatusInformation(percentComplete, statusLabel) {

    if(item.PERCENT_COMPLETE !== percentComplete) item.PERCENT_COMPLETE = percentComplete;
    if(item.STATUS_LABEL !== statusLabel) item.STATUS_LABEL = statusLabel;
    
}


// Save current date in defined field
function saveDate(field) {
    
    if(item[field] === null) item[field] = new Date();
    
}


// store approver user name upon review completion
function saveReviewer() {
    
    var user = Security.loadUser(userID);
    var userName = user.lastName + ", " + user.firstName;
    if(item.REVIEWED_BY !== userName) item.REVIEWED_BY = userName;
    if(item.COMMENTS === null) item.COMMENTS = item.workflowActions[0].comments;
    
}


// store date and comments of technical analysis completion
function saveTechnicalAnalysisDetails() {
    
    item.TECHNICAL_ANALYSIS_COMMENTS = item.workflowActions[0].comments;
    item.TECHNICAL_ANALYSIS_COMPLETION_DATE = new Date();
    
}


// initiate Change Request process
function createChangeRequest() {
    
    if(item.CHANGE_REQUEST !== null) return;
    
    createFollowUpProcess("WS_CHANGE_REQUESTS", "PROBLEM_REPORT", "CHANGE_REQUEST", "TITLE,DESCRIPTION,PRIORITY,SOURCE,AFFECTED_PRODUCT,AFFECTED_ITEM,AFFECTED_QUANTITY,ROOT_CAUSE,ROOT_CAUSE_CODE", true);
    
}



// initate CAPA process 
function initiateCAPA() {
    
    if(workspaceCAPA === null) return;
    if(workspaceCAPA === "") return;
    
    var processCAPA = createItem(workspaceCAPA);
    
    item.CAPA = processCAPA;
    item.CAPA_CREATION_DATE = new Date();
    
    if(linkCAPA !== null) {
        if(linkCAPA !== "") {
            processCAPA[linkCAPA] = item;
        }
    }
    
    if(fieldsCAPA === null) return;
    if(fieldsCAPA === "") return;
    
    for(var i = 0; i < fieldsCAPA.length; i++) {
        processCAPA[fieldsCAPA[i]] = item[fieldsCAPA[i]];
    }
    
}