// This script is part of the PRODUCT PORTFOLIO MANAGEMENT app available in the App Store
// Last modification date: May 17, 2021

switch(customTransID) {
    
    case 'NEW':
        reset();
        setMilestone(statesInWork[0], "ENTER", new Date(), null, 0, 0, true);  
        setDefaultMilestones();
        break;
        
    default:
        
        if(customTransID.indexOf("NEXT_MILESTONE_") === 0) {
            var index = Number(customTransID.replace(/NEXT_MILESTONE_/g, ""));
            item.CURRENT_CHECKLIST_DMSID = null;
            item.CURRENT_CHECKLIST_WSID = null;
            if(index > 0) nextMilestone(index);
            if(index === milestones.length) {
                archive();
            } else {
                createChecklist(index);
            }
        } else if(customTransID.indexOf("CANCEL_") === 0) {
            item.STEP = 99;
            archive();
        }
        break;
        
}


// At item creation set defaults & reset data in case of clones
function reset() {
    
    item.STEP = 0;
        
    resetField("PREVIOUS_PRODUCT_LINE", null);
    resetField("PREVIOUS_PRODUCT_CATEGORY", null);
    resetField("CURRENT_CHECKLIST_DMSID", null);
    resetField("CURRENT_CHECKLIST_WSID", null);
    
    item.PREVIOUS_PORTFOLIOS = [];
    
    for(var i = 1; i <= milestones.length; i++) {
        resetField("ACTUAL_DATE_" + i, null);
        resetField("CHECKLIST_"   + i, null);
    }
    
}
function resetField(fieldName, value) {
    if(item[fieldName] !== value) item[fieldName] = value;
}


// Set milestone data based on Product Line defaults
function setDefaultMilestones() {
    
    if(item.PRODUCT_LINE === null) return;
    
    for(var i = 1; i <= milestones.length; i++) setDefaultMilestone(i);
    
}
function setDefaultMilestone(index) {
    
    if(item["PLANNED_DATE_" + index] !== null) return;
    if(item.PRODUCT_LINE["MILESTONE_OFFSET_" + index] === null) return;
    
    var target = new Date();
    
    target.setDate(target.getDate() + item.PRODUCT_LINE["MILESTONE_OFFSET_" + index]);
    
    item["PLANNED_DATE_" + index] = target;
    
}


// Save actual date when progressing to next milestone
function nextMilestone(index) {
    item.STEP = index;
    item["ACTUAL_DATE_" + index] = new Date();
}


// Initiate new checklist
function createChecklist(index) {
    
    var add          = false;
    var elemTemplate = item.PROCESS_TEMPLATE;
    var targetDate   = item["PLANNED_DATE_" + (index + 1)];
    
    if(elemTemplate === null) return;
    
    for(var i = 0; i < elemTemplate.grid.length; i++) {
        if(elemTemplate.grid[i].PRODUCT_STATUS === item.descriptor.workflowState) {
            add = true;
            break;
        }
    }

    if(add === false) return;
    
    var elemChecklist = createItem("WS_CHECKLISTS");
        elemChecklist.PRODUCT = item;
        elemChecklist.PRODUCT_STATUS = item.descriptor.workflowState;
        elemChecklist.MILESTONE_DATE = targetDate;
        elemChecklist.APPROVER = item["APPROVER_" + (index + 1)];
        
    item["CHECKLIST_" + (index + 1)] = elemChecklist;
    elemChecklist.relationships.addRelated(item, "Cross-Reference", "Bi-Directional", "Product Checklist");
    elemChecklist.relationships.addRelated(elemTemplate, "Cross-Reference", "Uni-Directional", "Template");
        
    for(var i = 0; i < elemTemplate.grid.length; i++) {
        var srcRow = elemTemplate.grid[i];
        if(srcRow.PRODUCT_STATUS === item.descriptor.workflowState) {
            var newRow = elemChecklist.grid.addRow();
                newRow.TITLE = srcRow.TITLE;
                newRow.DETAILS = srcRow.DETAILS;
                newRow.EXPECTED_OUTCOME = srcRow.EXPECTED_OUTCOME;
                newRow.TARGET_DATE = targetDate;
        }
    }
        
}


// Archive related data and remove references when product get Cancelled
function archive() {
    
    item.deleteItem();
    item.CURRENT_CHECKLIST_DMSID = null;
    item.CURRENT_CHECKLIST_WSID = null;
    
    removeFromGrid(item.PROGRAM, "PRODUCT");
    removeFromGrid(item.PRODUCT_LINE, "PRODUCT");
    
    removeFromProject(item.PRODUCT_LINE);
    
    for(var i = 1; i <= milestones.length; i++) {
        var elemChecklist = item["CHECKLIST_" + i];
        if(elemChecklist !== null) elemChecklist.deleteItem();
    }
    
}