// This script is part of the PRODUCT PORTFOLIO MANAGEMENT app available in the App Store
// Last modification date: May 14, 2021

setNumber();
setApprovers();
setMilestones();
setClassRating();
setNextMilestone();
setProductInWork();
updateChecklists();
updateRelatedProject("PRODUCT_LINE", "PREVIOUS_PRODUCT_LINE");
updateRelatedProjects("PORTFOLIOS", "PREVIOUS_PORTFOLIOS");


// Set & update number
function setNumber() {
    
    var number = item.ID;
    
    if(item.PRODUCT_YEAR !== null) number = item.PRODUCT_YEAR + "-" + number;
    
    if(item.PRODUCT_LINE !== null) {
        if(item.PRODUCT_LINE.PRODUCT_NAME_PREFIX !== null) {
            number = item.PRODUCT_LINE.PRODUCT_NAME_PREFIX + "-" + number;
        }
    }
    
    if(item.NUMBER !== number) item.NUMBER = number;
    
}


// Set default approvers based on Product Line defaults
function setApprovers() {
    
    var ownerID = item.descriptor.ownerID;
    var owner   = Security.loadUser(ownerID);
    var user    = owner.lastName + ", " + owner.firstName;
    
    for(var i = 1; i <= milestones.length ; i++) setApprover(i, user);
}
function setApprover(index, user) {
    
    var approver = user;
    
    if(item.PRODUCT_LINE !== null) {
        if(item.PRODUCT_LINE["APPROVER_" + index] !== null) {
            approver = item.PRODUCT_LINE["APPROVER_" + index];
        }
    }
    
    if(item["APPROVER_" + index] !== approver) item["APPROVER_" + index] = approver;
    
}


// Set Product Class based on rating
function setClassRating() {
    
    var result = 0;
    
    for(var i = 0; i < weighting.length; i++) result += addClassRating(result, i);
    
    var productClass = "?";
    
         if(result >= 3.6) productClass = "C";
    else if(result >= 2.4) productClass = "B";
    else if(result >    0) productClass = "A";
    
    if(item.CLASS !== productClass) item.CLASS = productClass;
    
    
}
function addClassRating(result, i) {
 
    var value = item["ASSESSMENT_" + (i+1)];
    
    if(value === null) return -100;
    
    var temp = value.split(" ");
    
    return  (Number(temp[0]) * weighting[i] / 100);
    
}


// Set milestones in given tab
function setMilestones() {

    setMilestone(milestones[0], "ENTER", item.PLANNED_DATE_1, null, 7, 10, true, false);   
    setMilestone(milestones[1], "ENTER", item.PLANNED_DATE_2, null, 7, 90, true, false);   
    setMilestone(milestones[2], "ENTER", item.PLANNED_DATE_3, null, 7, 100, true, false);   
    setMilestone(milestones[3], "ENTER", item.PLANNED_DATE_4, null, 7, 100, true, false);   
    setMilestone(milestones[4], "ENTER", item.PLANNED_DATE_5, null, 7, 100, true, false);   
    
}


// Set next milestone date
function setNextMilestone() {
    
    var index = item.STEP + 1;
    
    if(item["PLANNED_DATE_" + index] !== null) {
        item.NEXT_MILESTONE_DATE = item["PLANNED_DATE_" + index];
    }
    
}


// Add product to grids of parents while being in work
function setProductInWork() {
    
    var status = item.descriptor.workflowState;
    
    if(statesInWork.indexOf(item.descriptor.workflowState) === -1)  {
        removeFromGrid(item.PRODUCT_CATEGORY, "PRODUCT");
        removeFromGrid(item.PRODUCT_LINE, "PRODUCT");
    } else {
        updateRelatedGrid("PRODUCT_CATEGORY", "PREVIOUS_PRODUCT_CATEGORY", "PRODUCT");
        updateRelatedGrid("PRODUCT_LINE", "PREVIOUS_PRODUCT_LINE", "PRODUCT");
    }
    
    if(item.PREVIOUS_PRODUCT_CATEGORY !== item.PRODUCT_CATEGORY) item.PREVIOUS_PRODUCT_CATEGORY = item.PRODUCT_CATEGORY;
    
}


// In case of changes to the approver and/or milestone dates, update the related Checklist
function updateChecklists() {

    for(var step = item.STEP + 1; step <= milestones.length; step++) {
        
        var elemChecklist = item["CHECKLIST_" + step];
        
        if(elemChecklist !== null) {
            if(item["PLANNED_DATE_" + step] !== null) {
                if(elemChecklist.MILESTONE_DATE !== item["PLANNED_DATE_" + step]) {
                    elemChecklist.MILESTONE_DATE = item["PLANNED_DATE_" + step];
                }
            }
            if(elemChecklist.APPROVER !== item["APPROVER_" + step]) elemChecklist.APPROVER = item["APPROVER_" + step];
        }
        
    }

}