// This script is part of the NEW PRODUCT INTRODUCTION app available in the App Store
// last modification date: Nov 17, 2022


switch(customTransID) {
    
    case 'START':
        reset();
        autoAssign();
        if(item.PHASE !== null) item.PHASE.project.addTask(item);
        break;

    case 'CONFIRM_TASK':
        saveDate("ACTUAL_START_DATE");
        addTaskInWork();
        break;
        
    case 'SEND_TO_REVIEW':
        item.PERCENT_COMPLETE = 100;
        updateParent(100);
        relateTaskToProject();
        break;

    case 'RETURN_FROM_REVIEW':
        item.PERCENT_COMPLETE = 75;
        break;
        
    default:
        if(customTransID.indexOf("SET_PROGRESS_") === 0) {
            var value             = customTransID.replace(/SET_PROGRESS_/g, "");
            item.PERCENT_COMPLETE = value;
            item.LAST_COMMENT     = item.workflowActions[0].comments;
            item.LAST_UPDATE      = new Date();
            updateParent(Number(value));
            relateTaskToProject();
        } else if(customTransID.indexOf("CANCEL_TASK_") === 0) {
            item.CANCELLATION_DATE = new Date();
            removeTaskInWork();
            removeTaskFromPhase();
            updateParent(100);
        } else if(customTransID.indexOf("COMPLETE_TASK_") === 0) {
            saveDate("ACTUAL_START_DATE");
            saveDate("ACTUAL_COMPLETION_DATE");
            item.PERCENT_COMPLETE = 100;
            item.LAST_COMMENT     = item.workflowActions[0].comments;
            item.LAST_UPDATE      = new Date();
            updateParent(100);
            removeTaskInWork();
            relateTaskToProject();
        }
        break;
    
}

//saveStatusLabel();
appendToThread("COMMENTS_LOG", true);


// Clear relevant fields in case of clone
function reset() {

    resetField("LAST_COMMENT", null);
    resetField("LAST_UPDATE", null);
    resetField("ACTUAL_START_DATE", null);
    resetField("ACTUAL_COMPLETION_DATE", null);
    resetField("COMMENTS_LOG", null);
    resetField("PERCENT_COMPLETE", 0);
    
    item.grid.clear();
    
}
function resetField(fieldName, value) {
    if(item[fieldName] !== value) item[fieldName] = value;
} 


// Assign task automatically if assignee is defined
function autoAssign() {
    
    if(item.ASSIGNEE !== null) { 
        item.performWorkflowTransitionByCustomID("ASSIGN_TASK", "Assigned by project lead"); 
    }
    
}


// Save current date in defined field if empty
function saveDate(fieldName) {
    
    if(item[fieldName] === null) item[fieldName] = new Date();
    
}


// Add / remove task to / from project tab of select assignee
function addTaskInWork() {
    
    if(item.ASSIGNEE === null) return;
    
    item.ASSIGNEE.project.addTask(item);
    
}
function removeTaskInWork() {
    
    var project = item.PROJECT;
    
    if(project !== null) {
        for(var i = 0; i < project.grid.length; i++) {
            if(project.grid[i].TASK === item) {
                project.grid[i].remove();
                break;
            }
        }
    }
    
    
    if(item.ASSIGNEE === null) return;
    
    var tasks = item.ASSIGNEE.project.children;
    
    for(var i = tasks.length - 1; i >= 0; i--) {
        var task = tasks[i];
        if(task !== null) {
            if(task.item !== null) {
                if(task.item.master.dmsID == dmsID) {
                    task.remove(); 
                    return;
                }
            }
        }
    }
    
}
function removeTaskFromPhase() {
        
    if(item.PHASE === null) return;
    
    var tasks = item.PHASE.project.children;
    
    for(var i = tasks.length - 1; i >= 0; i--) {
        var task = tasks[i];
        if(task !== null) {
            if(task.item !== null) {
                if(task.item.master.dmsID == dmsID) {
                    task.remove(); 
                    return;
                }
            }
        }
    }
    
}


// Append workflow comment to Comments Log
function appendToThread(field, append) {
    
    if(item.workflowActions[0].transition.fromState === null) return;
    if(item.workflowActions[0].comments === null) return;
    if(item.workflowActions[0].comments === "") return;
    
    var comment = item.workflowActions[0].comments;
    var date    = new Date();
    var thread  = "&lt;p&gt;";
    
    thread += "&lt;span style='font-size:0.8em;font-style:italic;color:#ffa600;white-space:nowrap;'&gt;";
    thread += item.workflowActions[0].transition.fromState.shortName + " &gt; " + item.workflowActions[0].transition.toState.shortName;
    thread += "&lt;/span&gt;";
    thread += "&lt;br&gt;";
   
    if(comment !== null) {
        if(comment !== "") {
            thread += comment + "&lt;br&gt;";
        }
    }
    
    thread += "&lt;span style='font-size:0.8em;font-style:italic;color:#999;white-space:nowrap;'&gt;";
    thread += getUserName() + " (" + date.toDateString() + ")";
    thread += "&lt;/span&gt;";
    thread += "&lt;/p&gt;";
            
    if(append) {
        if(item[field] !== null) thread += item[field];
    }
            
    item[field] = thread;
    
}


// If task belongs to Phase and is final task, move project to next status
function updateParent(newProgress) {
    
    var project         = item.PROJECT;
    var phase           = item.PHASE;
    var progress        = 0;
    var complete        = ((customTransID !== "SEND_TO_REVIEW") && (newProgress === 100));
    var indexPhase      = getPhaseIndex(phase.PHASE_NAME);
    var effortChange    = calcEffort();
    var effortActual    = (phase.ACTUAL_EFFORT === null) ? 0 : phase.ACTUAL_EFFORT;
    var durationTotal   = 0;
    var progressTotal   = 0;

    if(phase === null) return;
    
    var tasks = phase.project.children;
        
    for(var i = tasks.length - 1; i >= 0; i--) {
        
        var durationTask = Number(tasks[i].duration.split("d")[0]);
        var progressTask = 0;
        
        if(tasks[i].target.master.dmsID !== dmsID) {
            progressTask = tasks[i].progress * durationTask;
            // if(tasks[i].progress !== 100) complete = false;    
            if(tasks[i].status !== stateTaskCompleted) complete = false;    
        } else {
            progressTask = newProgress * durationTask;
        }
        
        progressTotal += progressTask
        durationTotal += durationTask;
    }
    
    if(complete) {

        if(phase.descriptor.workflowState === statePhaseInWork) {
            phase.performWorkflowTransitionByCustomID("TASKS_COMPLETE", "All tasks completed");
            phase.PERCENT_COMPLETE = 100;
            phase.ACTUAL_COMPLETION_DATE = new Date();
        }
        
        project["ACTUAL_COMPLETION_PHASE_" + indexPhase] = new Date();
        
        if(project.descriptor.workflowState === phase.PHASE_NAME) {
            project.CURRENT_PHASE_PERCENT_COMPLETE = 100;
            project.performWorkflowTransitionByCustomID("SUBMIT_TO_GATE_REVIEW_" + indexPhase, "Phase " + item.PHASE.PHASE_NAME + " completed");
        }
        
    } else {
        
        var progress = Math.round((progressTotal / durationTotal), 0);
        
        if(progressTotal === 0) { if(durationTotal === 0) {
            progress = 0;
        }}
        
        phase.PERCENT_COMPLETE = progress;
        project.CURRENT_PHASE_PERCENT_COMPLETE = progress;
        
    }

    effortActual += effortChange;
    
    phase.ACTUAL_EFFORT = effortActual;
    project["ACTUAL_EFFORT_PHASE_" + indexPhase] = effortActual;
    
}
function getPhaseIndex(phaseName) {
    
    for(var x = 0; x < phases.length; x++) {
        if(phases[x] === phaseName) {
            return (x + 1);
        }
    }
    
    return -1;
    
}
function calcEffort() {
    
    var effort = 0;
    
    for(var i = 0; i < item.grid.length; i++) {
        var add = item.grid[i].EFFORT;
        if(add === null) add = 0;
        effort += add;
    }
    
    
    var effortActual = (item.ACTUAL_EFFORT === null) ? 0 : item.ACTUAL_EFFORT;
    
    if(item.ACTUAL_EFFORT !== effort) item.ACTUAL_EFFORT = effort;
    
    return (effort - effortActual);
    
}


// Connect task to project to provide direct access to task attachments
function relateTaskToProject() {
    
    if(item.attachments.length > 0) {
        
        var isProjectConnected = false;
        var isPhaseConnected   = false;
        
        for(var i = 0; i < item.relationships.length; i++) {
            if(item.relationships[i].item === item.PROJECT) {
                isProjectConnected = true;
                continue;
            } else if(item.relationships[i].item === item.PHASE) {
                isPhaseConnected = true;
                continue;
            }
        }
        
        if(!isProjectConnected) { item.relationships.addRelated(item.PROJECT, 'Cross-Reference', 'Bidirectional', 'Task with deliverables');} 
        if(!isPhaseConnected)   { item.relationships.addRelated(item.PHASE,   'Cross-Reference', 'Bidirectional', 'Task with deliverables');} 

    }
}