// This script is part of the NEW PRODUCT INTRODUCTION app available in the App Store
// Last modification date: Apr 29, 2021

switch(customTransID) {
    
    case 'CREATE':
        reset();
        createPhases();
        break;
 
    case 'START_PROJECT':
        updateProjectSchedule(1);
        startPhase(1);
        setRequiredApprovers(1);
        break;
 
    case 'REJECT_PROJECT':
        item.STEP = 99;
        item.deleteItem();
        deletePhases();
        break;
        
    case 'CLOSE_PROJECT':
        item.STEP = 11;
        item.CURRENT_PHASE = null;
        item.CURRENT_PHASE_PERCENT_COMPLETE = null;
        updateGate(phases.length);
        break;
        
    default:
        
        if(customTransID.indexOf("APPROVE_") === 0) {
            
            var index = Number(customTransID.replace(/APPROVE_/g, ""));
            item.REMAINING_APPROVERS.remove(getUserName());
            
            // Upon final approval initiate next phase
            if(item.REMAINING_APPROVERS.length === 1) {
                item.performWorkflowTransitionByCustomID('START_PHASE_' + (index + 1), 'Gate approved');
                updateGate(index);
                updateProjectSchedule(index + 1);
                startPhase(index + 1);
                setRequiredApprovers(index + 1);
            }
            
        } else if(customTransID.indexOf("START_PHASE_") === 0) {
            
            // initiate next phase
            var index = customTransID.replace(/START_PHASE_/g, "");
            updateGate(index - 1);
            updateProjectSchedule(index);
            startPhase(index);
            setRequiredApprovers(index);
            
        } else if(customTransID.indexOf("SUBMIT_TO_GATE_REVIEW_") === 0) {
            
            // manually push from current phase to gate review
            var index = customTransID.replace(/SUBMIT_TO_GATE_REVIEW_/g, "");
            item["REVIEWS_COUNTER_" + index] = item["REVIEWS_COUNTER_" + index] + 1;
            item.STEP = item.STEP + 1;
            if(item.grid.length === 0) item["ACTUAL_COMPLETION_PHASE_" + index] = new Date();
            setRemainingApprovers(index);
            sendMail(index);

        } else if(customTransID.indexOf("INITIATE_ADDITIONAL_TASKS_") === 0) {

            // During Gate Review return to phase in order to initiate further tasks
            var index = customTransID.replace(/INITIATE_ADDITIONAL_TASKS_/g, "");
            item.STEP = item.STEP - 1;
            item["ACTUAL_COMPLETION_PHASE_" + index] = null;
            updatePhase();
            if(item.grid.length > 0) {
                updateProjectTasks();
                updateProjectScheduleFromPhase();
            }
            
        } else if(customTransID.indexOf("CANCEL_") === 0) {
            
            // cancellation of project from gate review
            item.STEP = 99;
            item.CURRENT_PHASE = null;
            item.CURRENT_PHASE_PERCENT_COMPLETE = null;
            for(var j = 0; j < item.grid.length; j++) if(item.grid[j].TASK !== null) { item.grid[j].TASK.deleteItem(); }
            item.deleteItem();
            deletePhases();
            item.grid.clear();
            
        }
        
        break;
    
}


// Reset values during item creation / clone
function reset() {
    
    item.grid.clear();
    item.STEP = 0;
    
    resetField("CURRENT_PHASE", null);
    resetField("CURRENT_PHASE_PERCENT_COMPLETE", null);
    
    for(var i = 1; i <= phases.length; i++) {
        resetField("REVIEWS_COUNTER_" + i, 0);   
    }
    
}
function resetField(fieldName, value) {
    if(item[fieldName] !== value) item[fieldName] = value;
}
function createPhases() {

    for(var i = 0; i < phases.length; i++) {

        var elemPhase                       = createItem(wsProjectPhases);
            elemPhase.PROJECT_NUMBER        = item.IDPRJ;
            elemPhase.PHASE_NAME            = phases[i];
            elemPhase.PROJECT               = item;
            elemPhase.PERCENT_COMPLETE      = 0;
        
        item["PHASE_" + (i+1)] = elemPhase;
        
    }

}
function deletePhases() {

    for(var i = 1; i <= phases.length; i++) {
        var elemPhase = item["PHASE_" + i];
        if(elemPhase !== null) elemPhase.deleteItem();
    }
    
}



// Initiate phase with tasks
function startPhase(index) {

    item.STEP = item.STEP + 1;
    item.CURRENT_PHASE = item["PHASE_".concat(index)];
    item.CURRENT_PHASE_PERCENT_COMPLETE = 0;
    
    startPhaseTasks(item.CURRENT_PHASE);
    
}
function startPhaseTasks(elemPhase) {
    
    elemPhase.project.clear();
    
    var targetEffort = 0;

    for(var i = 0; i < elemPhase.grid.length; i++) {

        var row         = elemPhase.grid[i];
        var effort      = (row.PLANNED_EFFORT === null) ? 0 : row.PLANNED_EFFORT;
        var elemTask    = createProjectTask(row, item, elemPhase, effort);

        targetEffort += effort;
        
        var inWorkEntry                         = item.grid.addRow();
            inWorkEntry.TASK                    = elemTask;
            inWorkEntry.TITLE                   = row.TITLE;
            inWorkEntry.DESCRIPTION             = row.DESCRIPTION;
            inWorkEntry.PRIORITY                = row.PRIORITY;
            inWorkEntry.PLANNED_START_DATE      = row.PLANNED_START_DATE;
            inWorkEntry.PLANNED_COMPLETION_DATE = row.PLANNED_COMPLETION_DATE;
            inWorkEntry.PLANNED_EFFORT          = effort;
            inWorkEntry.TEAM                    = row.TEAM;
            inWorkEntry.TEAM_LEADER             = row.TEAM_LEADER;
            inWorkEntry.ASSIGNEE                = row.ASSIGNEE;
            inWorkEntry.REVIEWER                = row.REVIEWER;
            inWorkEntry.PHASE                   = elemPhase.PHASE_NAME;

    }
    
    elemPhase.ACTUAL_START_DATE = new Date();
    elemPhase.performWorkflowTransitionByCustomID('START_PHASE', 'Phase started automatically');
    elemPhase.grid.clear();
    
    return targetEffort;

}


// Set approvers
function setRequiredApprovers(index) {
    
    for(var i = 0; i < item.TEMPLATE["REVIEWERS_GATE_" + index].length; i++) {
        item.REQUIRED_APPROVERS.add(item.TEMPLATE["REVIEWERS_GATE_" + index][i]);
    }
    
    item.ADDITIOINAL_APPROVERS = [];
    item.REMAINING_APPROVERS = [];
    
}
function setRemainingApprovers(index) {
    
    for(var i = 0; i < item.REQUIRED_APPROVERS.length; i++) {
        item.REMAINING_APPROVERS.add(item.REQUIRED_APPROVERS[i]);
    } 
    for(var i = 0; i < item.ADDITIONAL_APPROVERS.length; i++) {
        item.REMAINING_APPROVERS.add(item.ADDITIONAL_APPROVERS[i]);
    }
    
    if(item.REMAINING_APPROVERS.length === 0) {
        item.REMAINING_APPROVERS.add(getUserName());
    }
    
}
function sendMail(index) {

    var addressees = [];
    var to = "";

    var footer  = "&lt;br/&gt;&lt;br/&gt;&lt;hr style='border: 1px solid #bbb;'&gt;";
        footer += "&lt;span style='color:#bbb;'&gt;";
        footer += "This mail has been generated automatically. Please do not respond to this mail.&lt;br/&gt;";
        footer += "In case of questions, please get in touch with ";
        footer += "&lt;a href='mailto:&quot; + item.master.owner.email + &quot;?subject=&quot; + item.descriptor.descriptor + &quot;'&gt;" + item.master.owner.email + "&lt;/a&gt;";
        footer += "&lt;/span&gt;";
    
		    
    for(var i = 0; i < item.REQUIRED_APPROVERS.length; i++) {
        addressees.push(item.REQUIRED_APPROVERS[i]);
    }
    for(var j = 0; j < item.ADDITIONAL_APPROVERS.length; j++) {
        var name = item.ADDITIONAL_APPROVERS[j];
        if(addressees.indexOf(name) === -1) addressees.push(name);
    }
		
    for(var k = 0;  k < addressees.length; k++) {
		        
        var addressee   = addressees[k];
        var name        = addressee.split(", ");
        var userInfo    = Security.searchUsers({lastName:name[0], firstName:name[1]}); 
        var user        = Security.loadUser(userInfo[0].id);
		        
        to += "," + user.email;
		        
    }
    
    var email         = new Email();
		email.to      = to;
		email.subject = "Project is ready for Gate Review : " + item.descriptor.descriptor;
		email.body    = "Please review the given project in PLM which is listed in your My Outstanding Work list" + footer;
    email.send();
    
}


// Update project schedule when entering a new phase
function updateProjectSchedule(index) {
    
    if(item.UPDATE_SCHEDULE === false) return;

    var dateBase = new Date();
        dateBase = getNextWorkingDay(dateBase, 1);
    
    for(var i = index; i <= phases.length; i++) {
        dateBase = updatePhaseTasks(item, item["PHASE_" + i], dateBase, i);
    }
    
}


// Upon Gate Rejection reset Phase
function updatePhase() {
    
    var elemPhase = item.CURRENT_PHASE;
    
    if(elemPhase === null) return;
    
    if(elemPhase.ACTUAL_COMPLETION_DATE !== null) elemPhase.ACTUAL_COMPLETION_DATE = null;
    
    if(elemPhase.descriptor.workflowState === "Completed") elemPhase.performWorkflowTransitionByCustomID('INITIATE_ADDITIONAL_TASKS', 'Returned to phase during gate review');
    
}


// Upon Gate Approval update Gantt
function updateGate(index) {

    if(index > 0) {
        
        item["ACTUAL_COMPLETION_GATE_" + index] = new Date();
    
        if(item.UPDATE_SCHEDULE) {
        
            var now         = new Date();
            var reviewStart = getNextWorkingDay(new Date(item.workflowActions[0].timeStamp), 1);

            if(reviewStart.getTime() > now.getTime()) reviewStart = now;

            setProjectPhase(item, "G" + index, reviewStart, now, 100);
            
        } else {
         
            updateProjectPhase(item, "G" + index, 100);
            
        }
        
    }
    
}