// This script is part of the NEW PRODUCT INTRODUCTION app available in the App Store
// Last modification date: May 28, 2021

calcTotalCost();
copyProjectTemplate();
updateParentProject("PARENT_PROJECT", "PREVIOUS_PARENT_PROJECT");


// Calculate Total Project Budget
function calcTotalCost() {
    
    var costPlanned = 0;
    var costActual  = 0;
    
    for(var i = 1; i <= 5; i++) if(item["PLANNED_COST_" + i] !== null) costPlanned += item["PLANNED_COST_" + i];
    for(var j = 1; j <= 5; j++) if( item["ACTUAL_COST_" + j] !== null) costActual  += item["ACTUAL_COST_" + j];
    
    if(item.TOTAL_PLANNED_COST !== costPlanned) item.TOTAL_PLANNED_COST = costPlanned;
    if( item.TOTAL_ACTUAL_COST !== costActual ) item.TOTAL_ACTUAL_COST  = costActual;
    
}


// Copy tasks of selected template and compute grid
function copyProjectTemplate() {
    
    if(item.descriptor.workflowState !== "Planning") return;
    
    var template    = item.TEMPLATE;
    var dateBase    = item.START_DATE;
    
    if(template === item.PREVIOUS_TEMPLATE) {
        if(dateBase.getTime() === item.PREVIOUS_START_DATE.getTime()) {
            return;
        }
    }

    item.UPDATE_SCHEDULE     = template.UPDATE_SCHEDULE;
    item.PREVIOUS_TEMPLATE   = item.TEMPLATE;
    item.PREVIOUS_START_DATE = item.START_DATE;
    
    var userName = getUserName();
    var isNew    = (item.project.children.length < 1) ;
    
    for(var i = 1; i <= phases.length; i++) {
        
        item["DURATION_REVIEW_" + i] = template["DURATION_REVIEW_" + i];
        setApprover(template, userName, i);
        dateBase = setPhaseTasks(template, item["PHASE_" + i], phases[(i-1)], dateBase, i);
        if(isNew) item.project.addTask(item["PHASE_" + i]);
        
    }
    
}
function setApprover(template, userName, step) {
    
    var approver = (template["REVIEWER_GATE_" + step] === null) ? userName :  template["REVIEWER_GATE_" + step];
    
    if(item["APPROVER_" + step] !== approver) item["APPROVER_" + step] = approver;
    
}
function setPhaseTasks(template, elemPhase, phaseName, dateBase, step) {
    
    var tasks       = [];
    var min         = 1000000;
    var max         = -1;
    var phaseEnd    = new Date();
    var phaseStart  = getNextWorkingDay(dateBase, 0);
    var phaseEffort = 0;
    
    elemPhase.grid.clear();
    elemPhase.project.clear();
    
    for(var x = 0 ; x < template.grid.length; x++) {
    
        var row = template.grid[x];

        if(row.PHASE === phaseName) {
            
            var task = {
                "id"        : row.ID,
                "pre"       : row.PREDECESSOR,
                "duration"  : row.DURATION,
                "offset"    : 0,
                "row"       : x,
            };
    
            if(row.PREDECESSOR !== null) {
                if(row.PREDECESSOR > max) max = row.PREDECESSOR;
                if(row.PREDECESSOR < min) min = row.PREDECESSOR;
            } else {
                var endDate = getWorkingDaysEnd(phaseStart, row.DURATION);
                task.start  = new Date(phaseStart.getTime());
                task.end    = new Date(endDate.getTime());
            }
            
            phaseEffort += (row.EFFORT === null) ? 0 : row.EFFORT;
            tasks.push(task);
            
        }
    
    }
    
    for(var i = min; i <= max; i++) {
        
        var predEnd  = null;
        var newStart = new Date(phaseStart.getTime());
        
        for(var j = 0; j < tasks.length; j++) {
            if(i === tasks[j].id) {
                predEnd = tasks[j].end;
                break;
            }
        }
        
        if(predEnd !== null) newStart = getNextWorkingDay(predEnd, 1);
        
        for(var k = 0; k < tasks.length; k++) {
            if(i === tasks[k].pre) {
                var newEnd = getWorkingDaysEnd(newStart, tasks[k].duration);
                tasks[k].start = newStart;
                tasks[k].end = newEnd;
                if(predEnd === null) {
                    tasks[k].pre = null;
                }
            }
        }
        
    }
    
    for(var l = 0; l < tasks.length; l++) {
        
        var task    = tasks[l];
        var source  = template.grid[Number(task.row)];
        var rowNew  = elemPhase.grid.addRow();
        
        rowNew.ID                       = source.ID;
        rowNew.PREDECESSOR              = task.pre;
        rowNew.TITLE                    = source.TITLE;
        rowNew.DESCRIPTION              = source.DESCRIPTION;
        rowNew.PRIORITY                 = source.PRIORITY;
        rowNew.PLANNED_START_DATE       = task.start;
        rowNew.PLANNED_COMPLETION_DATE  = task.end;
        rowNew.PLANNED_DURATION         = task.duration;
        rowNew.PLANNED_EFFORT           = source.EFFORT;
        rowNew.TEAM                     = source.TEAM;
        rowNew.TEAM_LEADER              = source.TEAM_LEADER;
        rowNew.ASSIGNEE                 = source.ASSIGNEE;
        rowNew.REVIEWER                 = source.REVIEWER;
            
        elemPhase.project.addTask(source.TITLE, task.start, task.end);
  
        if(task.end.getTime() > phaseEnd.getTime()) phaseEnd = new Date(task.end.getTime());
        
    }
    
    elemPhase.PLANNED_START_DATE        = phaseStart;
    elemPhase.PLANNED_COMPLETION_DATE   = phaseEnd;
    elemPhase.PLANNED_EFFORT            = phaseEffort;

    item["PLANNED_EFFORT_PHASE_"     + step] = phaseEffort;
    item["PLANNED_COMPLETION_PHASE_" + step] = phaseEnd;
    
    var buffer = item["DURATION_REVIEW_" + step];
    
    if((buffer === 0) || (buffer === null)) {
        
        item["PLANNED_COMPLETION_GATE_"  + step] = phaseEnd;

        return getNextWorkingDay(phaseEnd, 1);
        
    } else {
        
        var reviewStart = getNextWorkingDay(phaseEnd, 1);
        var reviewEnd   = getWorkingDaysEnd(reviewStart, buffer);
        
        setProjectPhase(item, "G" + step, reviewStart, reviewEnd, 0);   
        
        item["PLANNED_COMPLETION_GATE_"  + step] = reviewEnd;
        
        return getNextWorkingDay(reviewEnd, 1);
        
    }

}


// Connect to parent project if selected
function updateParentProject(fieldNew, fieldOld) {
    
    var projectNew = item[fieldNew];
    var projectOld = item[fieldOld];
    
    if(projectNew === projectOld) return;
    
    if(projectNew !== null) projectNew.project.addTask(item);
    
    if(projectOld !== null) {

        var tasks = projectOld.project.children;
        
        for(var i = tasks.length - 1; i >= 0; i--) {
            var task = tasks[i];
            if(task !== null) {
                if(task.item !== null) {
                    var taskId = task.item.master.dmsID;
                    if(taskId == dmsID) {
                        task.remove(); 
                        return;
                    }
                }
            }
        }

    }
    
    item[fieldOld] = projectNew;
    
}