// This script is part of the QUICK RELEASES app available in the App Store
// last modification date: Mar 23, 2021


var tenant              = "tenant-to-be-defined-in-script-quick-releases-workflow-actions";
var statusUnreleased    = "Unreleased";
var actionPreRelease    = "To Pre-Release";
var actionNewVersion    = "Design Revision";


switch(customTransID) {
    
    case 'START':
        reset();
        break;
        
    case 'SET_DEFAULT_LIFECYCLE_ACTIONS':
        setLifecycleActions();
        break;
    
    case 'PERFORM_QUICK_RELEASE':
    case 'APPROVE':
        var user = Security.loadUser(userID);
        item.COMPLETED_BY = getUserName(user);
        item.COMPLETION_DATE = new Date();
        item.COMPLETION_COMMENTS = item.workflowActions[0].comments;
        sendNotification(user);
        break;
    
}


// Reset read-only fields in case of a clone
function reset() {

    item.COMPLETED_BY        = null;
    item.COMPLETION_DATE     = null;
    item.COMPLETION_COMMENTS = null;

}


// Set default lifecycle action for all managed items (to reach preliminary release status)
function setLifecycleActions() {
    
    for(var i = 0; i < item.workflowItems.length; i++) {
        if(item.workflowItems[i].lifecycle === null) {
            if(item.workflowItems[i].item.descriptor.dvi.lifecycleState.stateName === statusUnreleased) item.workflowItems[i].lifecycle = actionPreRelease;
            else  item.workflowItems[i].lifecycle = actionNewVersion;
        }
    }
    
}


// Send notification to users selected in Distribution List field
function sendNotification(user) {
    
    if(item.DISTRIBUTION_LIST === null) return;
    if(item.DISTRIBUTION_LIST.length === 0) return;

    var to          = getAddressees();
    var styleHead   = "style='background:#ddd;text-align:left;'";
    var style       = "style='border-bottom:1px solid #ddd;'";
    
    if(to === "") return;
    
    var link = "https://";
        link += tenant;
        link += ".autodeskplm360.net/workspace#";
        link += "workspaceid=" + workspaceID;
        link += "&dmsid=" + dmsID;

    var header = "The following (" + item.workflowItems.length + ") items got released by ";
        header += "&lt;a href='&quot; + link + &quot;'&gt;" + item.descriptor.descriptor + "&lt;/a&gt;";
        header += "&lt;br/&gt;&lt;/br&gt;";

    var comments = getComments(user);

    var list  = "&lt;table cellpadding='8' cellspacing='0'&gt;";
        list += "&lt;tr&gt;&lt;th &quot; + styleHead + &quot;&gt;Item&lt;/th&gt;";
        list += "&lt;th &quot; + styleHead + &quot;&gt;Lifecycle Change&lt;/th&gt;";
        list += "&lt;th &quot; + styleHead + &quot;&gt;Comments&lt;/th&gt;&lt;/tr&gt;";
        
        for(var j = 0; j < item.workflowItems.length; j++) {
            
            var lifecycle = (item.workflowItems[j].lifecycle === null) ? "-" : item.workflowItems[j].lifecycle; 
            var comment   = (item.workflowItems[j].COMMENTS  === null) ? "-" : item.workflowItems[j].COMMENTS; 
            
            list += "&lt;tr&gt;";
            list += "&lt;td &quot; + style + &quot;&gt;" + item.workflowItems[j].descriptor + "&lt;/td&gt;";
            list += "&lt;td &quot; + style + &quot;&gt;" + lifecycle + "&lt;/td&gt;";
            list += "&lt;td &quot; + style + &quot;&gt;" + comment + "&lt;/td&gt;";
            list += "&lt;/tr&gt;";
            
        }
        
        list += "&lt;/table&gt;";

    var footer  = "&lt;br/&gt;&lt;br/&gt;&lt;hr style='border: 1pt solid #bbb;'&gt;";
        footer += "&lt;span style='color:#bbb;'&gt;";
        footer += "This mail has been generated automatically. Please do not respond to this mail.&lt;br/&gt;";
        footer += "In case of questions, please get in touch with ";
        footer += "&lt;a href='mailto:&quot; + item.master.owner.email + &quot;?subject=&quot; + item.descriptor.descriptor + &quot;'&gt;" + item.master.owner.email + "&lt;/a&gt;";
        footer += "&lt;/span&gt;";

	var email           = new Email();
	    email.to        = to;
		email.subject   = "Quick Release Notification (" + item.descriptor.descriptor + ")";
		email.body      = header + comments + list + footer;
        email.send();
        
}
function getAddressees() {
		
	var to = "";
		
    for(var i = 0;  i < item.DISTRIBUTION_LIST.length; i++) {
		        
        var addressee   = item.DISTRIBUTION_LIST[i];
        var name        = addressee.split(", ");
        var userInfo    = Security.searchUsers({lastName:name[0], firstName:name[1]}); 
        var user        = Security.loadUser(userInfo[0].id);
		    
        if(to !== "") to += ",";
		        
        to += user.email;
		        
    }
    
    return to;
    
}
function getComments(user) {
    
    var comments = item.workflowActions[0].comments;
    
    if(comments === null) {
        return "";
    } else if(comments === "") {
        return "";
    } else {
        comments = "Comments of " + user.lastName + ", " + user.firstName + " : " + comments + "&lt;br/&gt;&lt;br/&gt;";
    }
    
    return comments;
    
}


function getUserName(user) {
    return user.lastName + ", " + user.firstName;
}