// This script is part of the REQUIREMENTS MANAGEMENT app available in the App Store
// Last modification date: Mar 21, 2023


switch(customTransID) {

    case "START":
        reset();    
        setOwnership();
        break;

    case "FINISH_PREPARATION":
        saveActuals(1);
        break;

    case "SET_ALL_TEST_RESULTS_PENDING":
        setAllTestResults("Pending");
        break;
        
    case "SET_ALL_TEST_RESULTS_SUCCESS":
        setAllTestResults("Success");
        break;
        
    case "SET_ALL_TEST_RESULTS_FAILED":
        setAllTestResults("Failed");
        break;

    case "FINISH_EXECUTION":
        saveActuals(2);
        setTestResult();
        break;

    case "SUBMIT_FOR_ACKNOWLEDGEMENT":
    case "CLOSE_1":
        saveActuals(3);
        updateRequirements();
        triggerRepetition();
        break;

    case "ACKNOWLEDGE":
        saveAcknowledgement();
        break;

}


function reset() {

    if(item.IS_REPETITION === true) {
        item.IS_REPETITION = false;
        copyPreviousTestResult();
    } else {
        item.REPETITION = 1;
        setNumber();
        resetFields();
    }
    
}

function setOwnership() {
    
    if(item.REQUIREMENT_SPECIFICATION === null) return;
    if(item.REQUIREMENT_SPECIFICATION.GROUP_NAME === null) return;
    if(item.REQUIREMENT_SPECIFICATION.GROUP_NAME === "") return;
    
    var group = Security.loadGroup(item.REQUIREMENT_SPECIFICATION.GROUP_NAME);
    
    if(group !== null) item.master.groupAdditionalOwners.push(group);
    
}


function saveActuals(i) {
    
    item['ACTUAL_DATE_' + i] = new Date();
    item['COMMENTS_' + i] = item.workflowActions[0].comments;
    
}


function copyPreviousTestResult() {

    if(item.PREVIOUS_VALIDATION === null) return;

    for(var i = 0; i < item.PREVIOUS_VALIDATION.grid.length; i++) {

        var source = item.PREVIOUS_VALIDATION.grid[i];
        var target = item.grid.addRow();

        target.STEP                 = source.STEP;
        target.TEST_PROCEDURE       = source.TEST_PROCEDURE;
        target.EXPECTED_RESULT      = source.EXPECTED_RESULT;
        target.SUCCESS_CRITERIA     = source.SUCCESS_CRITERIA;
        target.BASE_REQUIREMENT     = source.BASE_REQUIREMENT;
        target.RELATED_REQUIREMENTS = source.RELATED_REQUIREMENTS;
        target.RESULT               = 'Pending';
        target.PREVIOUS_RESULT      = source.RESULT;
        
        if(source.BASE_REQUIREMENT !== null) source.BASE_REQUIREMENT.VALIDATION_PROCESS = item;
        
    }

}
function setNumber() {

    var prefix  = 'RV-';
    var seq     = Sequencer.get('Requirement Validations');
    var next    = seq.nextValue();
    var number  = ('000000' + next).slice(-6);
	    
    item.ID = prefix + number;

}
function resetFields() {

    resetField('TARGET_DATE_1');
    resetField('TARGET_DATE_2');
    resetField('TARGET_DATE_3');
    resetField('ACTUAL_DATE_1');
    resetField('ACTUAL_DATE_2');
    resetField('ACTUAL_DATE_3');
    resetField('COMMENTS_1');
    resetField('COMMENTS_2');
    resetField('COMMENTS_3');
    resetField('RESULT');
    resetField('REPETITION_REQUIRED');
    resetField('REPETITION_DATE');
    resetField('PREVIOUS_VALIDATION');
    resetField('NEXT_VALIDATION');

    item.ACKNOWLEDGEMENT_COMPLETED =  [];

}
function resetField(fieldId) {
    if(item[fieldId] !== null) item[fieldId] = null;
}

function getUserName() {
    var usr = Security.loadUser(userID);
    return usr.lastName + ", " + usr.firstName;
}

function setAllTestResults(result) {
    for(var i = 0; i < item.grid.length; i++) item.grid[i].RESULT = result;
}


function setTestResult() {

    var testResult = 4;

    for(var i = 0; i < item.grid.length; i++) {

        switch(item.grid[i].RESULT) {

            case "Pending":
                testResult = 1;
                break;

            case "Failed":
                if(testResult > 2) testResult = 2;
                break;

            case "Conditional Success":
                if(testResult > 3) testResult = 3;
                break;
        }

    }

    switch(testResult) {

        case 1: 
            item.RESULT = "Pending";
            break;

        case 2: 
            item.RESULT = "Failed";
            break;

        case 3: 
            item.RESULT = "Conditional Success";
            break;

        case 4: 
            item.RESULT = "Success";
            break;

    }

}


function updateRequirements() {

    var requirements = [];
    var results      = [];

    for(var i = 0; i < item.grid.length; i++) {

        if(item.grid[i].BASE_REQUIREMENT !== null) {
            storeRequirement(requirements, results, item.grid[i].BASE_REQUIREMENT, item.grid[i].RESULT);
        }
        if(item.grid[i].RELATED_REQUIREMENTS !== null) {
            for(var j = 0; j < item.grid[i].RELATED_REQUIREMENTS.length; j++) {
                storeRequirement(requirements, results, item.grid[i].RELATED_REQUIREMENTS[j], item.grid[i].RESULT);
            }
        }

    }

    for(var x = 0; x < requirements.length; x++) {

        requirements[x].LATEST_VALIDATION_RESULT    = results[x];
        requirements[x].LATEST_VALIDATION_DATE      = new Date();

        var row = requirements[x].grid.addRow();
            row.VALIDATION_PROCESS  = item;
            row.CLOSING_DATE        = new Date();
            row.RESULT              = results[x];
            row.REPETITION          = item.REPETITION_REQUIRED;
            row.PERFORMED_BY        = item.ASSIGNEE_2;

    }

}
function storeRequirement(requirements, results, requirement, result) {
    
    var index = requirements.indexOf(requirement);
    
    if(index === -1) {
        requirements.push(requirement);
        results.push(result);
    } else {
        if(results[index] !== 'Failed') {
            if(result === 'Failed') { results[index] = result; }
            else if(result === 'Conditional Success') { results[index] = result; }
        }
    }
}


function triggerRepetition() {

    if(item.REPETITION_REQUIRED === null) return;
    if(item.REPETITION_REQUIRED === 'No') return;

    var repetition = item.REPETITION + 1;
    
    var elemFollowup = createItem("WS_REQUIREMENT_VALIDATIONS");
        elemFollowup.ID = item.ID;
        elemFollowup.TITLE = item.TITLE;
        elemFollowup.REQUIREMENT_SPECIFICATION = item.REQUIREMENT_SPECIFICATION;
        elemFollowup.SPECIAL_INSTRUCTIONS = item.SPECIAL_INSTRUCTIONS;
        elemFollowup.REPETITION = repetition;
        elemFollowup.PREVIOUS_VALIDATION = item;
        elemFollowup.IS_REPETITION = true;

        elemFollowup.ASSIGNEE_1 = item.ASSIGNEE_1;
        elemFollowup.ASSIGNEE_2 = item.ASSIGNEE_2;
        elemFollowup.ASSIGNEE_3 = item.ASSIGNEE_3;

        elemFollowup.TARGET_DATE_2 = item.REPETITION_DATE;        
        
    item.NEXT_VALIDATION = elemFollowup;

}


function saveAcknowledgement() {

    var userName = getUserName();

    item.ACKNOWLEDGEMENT_REQUESTED.remove(userName);
    item.ACKNOWLEDGEMENT_COMPLETED.add(userName);

    if(item.ACKNOWLEDGEMENT_REQUESTED.length === 1) item.performWorkflowTransitionByCustomID("CLOSE_2", "Acknowledgement complete");

}