// This script is part of the SUPPLIER COLLABORATION app available in the App Store
// Last modification date: Apr 20, 2023

item.WORKFLOW_STATUS = item.descriptor.workflowState;

switch(customTransID) {
    
    case 'START':
        reset();
        copyTemplate();
        item.STEP = 1;
        break;
            
    case 'SUBMIT_TO_SUPPLIER':
        item.STEP = 2;
        item.SUBMITTAL_DATE = new Date();
        var supplierGroup = grantSupplierAccess(item, item);
        publishTemplate(supplierGroup);
        publishItems(supplierGroup);
        createDeliverables();
        break;
            
    case 'CONFIRM':
    case 'SEND_UPDATE':
        if(item.ACTUAL_DATE_1 === null) item.ACTUAL_DATE_1 = new Date();
        item.STEP = 3;
        break;  
            
    case 'RESUBMIT_PACKAGE':
        item.STEP = 3;
        break;  
            
    case 'RETURN_TO_REQUESTOR':
        item.STEP = 4;
        break; 
       
    case 'PROCESSING_STARTED':
    case 'CONTINUE_PROCESSING':
        item.STEP = 5;
        resetOnHoldDetails();
        break;        
            
    case 'SET_ON_HOLD_1':
    case 'SET_ON_HOLD_2':
        item.STEP = 6;
        item.ON_HOLD_SET_BY = getUserName();
        item.ON_HOLD_DATE = new Date();
        item.ON_HOLD_COMMENTS = item.workflowActions[0].comments;
        break; 

    case 'CONFIRM_SHIPMENT_1':
    case 'CONFIRM_SHIPMENT_2':
    case 'CONFIRM_SHIPMENT_3':
    case 'CONFIRM_SHIPMENT_4':
        resetOnHoldDetails();
        if(item.ACTUAL_DATE_2 === null) item.ACTUAL_DATE_2 = new Date();
        item.STEP = 7;
        break; 

    case 'CANCEL_1':
    case 'CANCEL_2':
    case 'CANCEL_3':
    case 'CANCEL_4':
        item.STEP = 99;
        item.deleteItem();
        clearItemPendingPackages();
        clearSupplierPendingPackages();
        break;

    case 'END_PROCESS_1':
    case 'END_PROCESS_2':
    case 'END_PROCESS_3':
    case 'END_PROCESS_4':
    case 'END_PROCESS_5':
    case 'END_PROCESS_6':
        item.COMPLETION_DATE = new Date();
        item.ACTUAL_DATE_3 = new Date();
        item.STEP = 20;
        clearItemPendingPackages();
        clearSupplierPendingPackages();
        break;

    case 'DELIVERY_RECEIVED':
        if(item.ACTUAL_DATE_3 === null) item.ACTUAL_DATE_3 = new Date();
        item.STEP = 8;
        break;

    case 'START_IGI':
        item.STEP = 9;
        break;

    case 'ISSUES_DETECTED':
        item.STEP = 10;
        break;        
        
}
    
    
function reset() {
    
    var seq = new Sequencer('Supplier Packages');
    
    item.NUMBER = prefixPackage + ('000000' + seq.nextValue().toString()).slice(-6);
    
    item.NEW_ITEMS_COUNTER      = null;
    item.MODIFIED_ITEMS_COUNTER = null;
    item.REPEATED_ITEMS_COUNTER = null;
    
    item.TOTAL_DELIVERABLES     = null;
    item.PENDING_DELIVERABLES   = null;
    item.NEXT_DELIVERABLE       = null;
    
    item.ACTUAL_DATE_1 = null;
    item.ACTUAL_DATE_2 = null;
    item.ACTUAL_DATE_3 = null;
    
    item.SUBMITTAL_DATE  = null;
    item.COMPLETION_DATE = null;

    item.ON_HOLD_SET_BY     = null;
    item.ON_HOLD_COMMENTS   = null;
    item.ON_HOLD_DATE       = null;
    
    for(var i = 1; i <= deliverablesCount; i++) item['DELIVERABLE_' + i] = null;

    item.relationships.clear();

    item.master.additionalOwners.length = 0;
    item.master.groupAdditionalOwners.length = 0;

}



function copyTemplate() {

    if(item.TEMPLATE === null) return;

    item.relationships.addRelated(item.TEMPLATE, "Cross-Reference", "Uni-Directional", "Supplier Package Template");

    copyTemplateField('TYPE');
    copyTemplateField('TITLE');
    copyTemplateField('REMARKS');
    
    copyTemplateField('FTP_USERNAME');
    copyTemplateField('FTP_PASSWORD');

    setTargetDate('TARGET_DATE_1', 'OFFSET_CONFIRMATION');
    setTargetDate('TARGET_DATE_2', 'OFFSET_SHIPMENT');
    setTargetDate('TARGET_DATE_3', 'OFFSET_ARRIVAL');

    copyTemplateDeliverable('1');
    copyTemplateDeliverable('2');
    copyTemplateDeliverable('3');
    copyTemplateDeliverable('4');
    copyTemplateDeliverable('5');


}
function copyTemplateField(fieldId) {

    if(item[fieldId] === null) {
        item[fieldId] = item.TEMPLATE[fieldId];
    }

}
function setTargetDate(fieldIdTarget, fieldIdOffset) {

    if(item[fieldIdTarget] !== null) return;

    var offset = item.TEMPLATE[fieldIdOffset];

    if(offset === null) return;

    var target = new Date();
        target.setDate(target.getDate() + offset);

    item[fieldIdTarget] = target;

}
function copyTemplateDeliverable(index) {

    copyTemplateField('DELIVERABLE_' + index + '_TITLE');
    copyTemplateField('DELIVERABLE_' + index + '_DETAILS');
    copyTemplateField('DELIVERABLE_' + index + '_REFERENCE_FILE');

    setTargetDate('DELIVERABLE_' + index + '_TARGET_DATE', 'DELIVERABLE_' + index + '_OFFSET');

}


function publishTemplate(supplierGroup) {

    if(item.TEMPLATE === null) return;

    grantSupplierGroupAccess(item.TEMPLATE, supplierGroup);

}
function publishItems(supplierGroup) {

    var supplierUser = getSupplierUser();
    var supplier     = item.SUPPLIER;

    if(typeof supplier === 'object') {
        supplier = item.SUPPLIER.descriptor.descriptor;
        if(typeof item.SUPPLIER.PENDING_PACKAGES !== 'undefined') {
            item.SUPPLIER.PENDING_PACKAGES.add(item);
        }
    }
    
    for(var i = 0; i < item.grid.length; i++) {
        
        var elemItem = item.grid[i].ITEM;
        
        if(elemItem !== null) {

            var flag = item.grid[i].FLAG_LABEL

            if(flag === 'New') grantSupplierGroupAccess(elemItem, supplierGroup);

            grantSupplierUserAccess(elemItem, supplierUser);

            item.relationships.addRelated(elemItem, 'Cross-Reference', 'Bi-Directional', supplier + ' | ' + elemItem.descriptor.descriptor);
                
        }
        
    }
    
}
function getSupplierUser() {

    var contact = item.SUPPLIER_CONTACT_PERSON;

    if(contact !== null) {

        var userSelected    = contact.split(',');
        var lastName        = userSelected[0].trim();
        var firstName       = userSelected[1].trim();
        var userSearch      = Security.searchUsers({lastName:lastName, firstName:firstName}); 

        if(userSearch !== null) return Security.loadUser(userSearch[0].id);

    }

    return null;

}
function grantSupplierGroupAccess(elemItem, supplierGroup) {
    
    if(supplierGroup === null) return;

    var additionalGroups = elemItem.master.groupAdditionalOwners;    
        additionalGroups.push(supplierGroup);


}
function grantSupplierUserAccess(elemItem, supplierUser) {

    if(supplierUser === null) return;

    var owners = elemItem.master.additionalOwners;
        owners.push(supplierUser);

}



function createDeliverables() {
    
    var date        = new Date();
    var elemNext    = null;
    var count       = 0;
    
    date.setDate(date.getDate() + 10000);
    
    for(var i = 1; i <= deliverablesCount; i++) {
        
        var deliverableTitle = item['DELIVERABLE_' + i + '_TITLE'];
        var deliverableDate  = item['DELIVERABLE_' + i + '_TARGET_DATE'];
        
        if((deliverableTitle !== null) && (deliverableTitle !== '')) {
        
            var elemDeliverable                 = createItem('WS_SUPPLIER_DELIVERABLES');
                elemDeliverable.TITLE           = deliverableTitle;
                elemDeliverable.DETAILS         = item['DELIVERABLE_' + i + '_DETAILS'];
                elemDeliverable.REFERENCE_FILE  = item['DELIVERABLE_' + i + '_REFERENCE_FILE'];
                elemDeliverable.SUPPLIER        = item.SUPPLIER;
                elemDeliverable.PACKAGE         = item;
    
            item.project.addTask(elemDeliverable);
            item.relationships.addRelated(elemDeliverable, 'Cross-Reference', 'Bi-Directional', 'Supplier Deliverable of Supplier Package');
            item['DELIVERABLE_' + i] = elemDeliverable;
            
            count++;

            if((deliverableDate !== null) && (deliverableDate !== '')) {
                elemDeliverable.TARGET_DATE = deliverableDate;
                if(deliverableDate.getTime() < date.getTime()) {
                    date.setDate(deliverableDate.getTime());
                    elemNext = elemDeliverable;
                }
            }
            
        }
                
    }
    
    item.TOTAL_DELIVERABLES   = count;
    item.PENDING_DELIVERABLES = count;
    
    if(elemNext !== null) item.NEXT_DELIVERABLE = elemNext;
    
}



function resetOnHoldDetails() {
    
    if(item.ON_HOLD_SET_BY   !== null) item.ON_HOLD_SET_BY   = null;
    if(item.ON_HOLD_DATE     !== null) item.ON_HOLD_DATE     = null;
    if(item.ON_HOLD_COMMENTS !== null) item.ON_HOLD_COMMENTS = null;
    
}



function clearItemPendingPackages() {

    if(item.SHARED_ITEMS.length === 0) return;

    if(typeof item.SHARED_ITEMS[0].PENDING_PACKAGES === 'undefined') return;

    for(var iClear = 0; iClear < item.SHARED_ITEMS.length; iClear++) {
        item.SHARED_ITEMS[iClear].PENDING_PACKAGES.remove(item);
    }

}
function clearSupplierPendingPackages() {

    if(item.SUPPLIER === null) return;

    if(typeof item.SUPPLIER.PENDING_PACKAGES === 'undefined') return;

    item.SUPPLIER.PENDING_PACKAGES.remove(item);

}