// This script is part of the SUPPLIER COLLABORATION app available in the App Store
// Last modification date: Apr 14, 2023


setContactDetails('REQUESTOR', item.REQUESTOR);
setContactDetails('SUPPLIER', item.SUPPLIER_CONTACT_PERSON);

setMilestone('Confirmed', 'ENTER', item.TARGET_DATE_1, 0,  10, true);
setMilestone('Shipped',   'ENTER', item.TARGET_DATE_2, 0,  80, true);
setMilestone('Shipped',   'EXIT' , item.TARGET_DATE_3, 0, 100, true);

setSharedItems();


function setContactDetails(prefix, user) {
    
    var userData, userAccount, userInfo;
    
    if(user === null) {
        
        if(prefix === 'REQUESTOR') {
            userData = Security.loadUser(userID);
            item[prefix] = userData.lastName + ", " + userData.firstName;
        } else if(typeof item.SUPPLIER === 'object') {
            if(typeof item.SUPPLIER.PRIMARY_CONTACT !== 'undefined') {
                if(item.SUPPLIER.PRIMARY_CONTACT !== null) {
                    if(item.SUPPLIER.PRIMARY_CONTACT !== '') {
                        userAccount  = item.SUPPLIER.PRIMARY_CONTACT.split(", ");
                        userInfo     = Security.searchUsers({lastName:userAccount[0], firstName:userAccount[1]}); 
                        userData     = Security.loadUser(userInfo[0].id);
                        item.SUPPLIER_CONTACT_PERSON = userData.lastName + ", " + userData.firstName;
                    }                
                }                
            }
        }
        
    } else {
        
        userAccount = user.split(", ");
        userInfo    = Security.searchUsers({lastName:userAccount[0], firstName:userAccount[1]}); 
        userData    = Security.loadUser(userInfo[0].id);
        
    }
    
    if(typeof userData === 'undefined') {
        
        item[prefix + '_MAIL_ADDRESS']  = null;
        item[prefix + '_ADDRESS']       = null;
        item[prefix + '_CITY']          = null;
        item[prefix + '_TIME_ZONE']     = null;
        
    } else {
    
    	if(item[prefix + '_MAIL_ADDRESS'] !== userData.email) {
    
            var address     = (userData.address1 === null) ? '' : userData.address1;
            var city        = (userData.city === null) ? '' : userData.city;
           
            if(userData.address2 !== null) address += ' ' + userData.address2; 
            if(userData.postal !== null) city = userData.postal + ' ' + city; 
    

    
		    item[prefix + '_MAIL_ADDRESS']  = userData.email;
		    item[prefix + '_ADDRESS']       = address;
		    item[prefix + '_CITY']          = city;
		    item[prefix + '_TIME_ZONE']     = userData.userTimeZone;
		    
		}
    
    }
    
}



function setSharedItems() {

    if(item.STEP > 1) return; 

    var counters     = [0,0,0];
    var supplier     = item.SUPPLIER;
    var gridItems    = [];
    var gridFlags    = [];
    
    if(typeof supplier === 'object') supplier = item.SUPPLIER.descriptor.descriptor;
    
    for(var y = item.grid.length -1; y >= 0; y--) {
            
        var elemGridItem = item.grid[y];
        
        if(item.SHARED_ITEMS.indexOf(elemGridItem.ITEM) < 0){
            if(elemGridItem.ITEM !== null) {
                if(typeof elemGridItem.ITEM.PENDING_PACKAGES !== 'undefined') {
                    elemGridItem.ITEM.PENDING_PACKAGES.remove(item);
                }
            }
            item.grid[y].remove();
        } else {
            gridItems.push(item.grid[y].ITEM);
            gridFlags.push(item.grid[y].FLAG_LABEL);
        }

    } 
    
    for(i = 0; i < item.SHARED_ITEMS.length; i++) {
        
        var elemItem    = item.SHARED_ITEMS[i];
        var index       = gridItems.indexOf(elemItem);
        var fileName    = elemItem.PRIMARY_FILE_NAME;
        var fileVersion = elemItem.PRIMARY_FILE_REVISION_VERSION;
        var fileId      = getFileID(elemItem, fileName);
        var flag        = 'New';
        
        if(index < 0) {

            var row  = item.grid.addRow({ITEM:elemItem, FILE_NAME:fileName, FILE_VERSION:fileVersion, ATTACHMENT_ID:fileId, ITEM_DMS_ID:elemItem.master.dmsID});
            
            flag = getFlagLabel(elemItem, row, supplier);
            row.FLAG_LABEL = flag;

            if(typeof elemItem.PENDING_PACKAGES !== 'undefined') {
                elemItem.PENDING_PACKAGES.add(item);
            }

        } else {
            flag = gridFlags[index];
        }

             if(flag === 'New')    counters[0]++;
        else if(flag === 'Update') counters[1]++;
        else                       counters[2]++;

    }

    if(item.NEW_ITEMS_COUNTER      !== counters[0]) item.NEW_ITEMS_COUNTER      = counters[0];
    if(item.MODIFIED_ITEMS_COUNTER !== counters[1]) item.MODIFIED_ITEMS_COUNTER = counters[1];
    if(item.REPEATED_ITEMS_COUNTER !== counters[2]) item.REPEATED_ITEMS_COUNTER = counters[2];
    
    item.TOTAL_ITEMS_COUNTER = counters[0] + counters[1] + counters[2];

}
function getFileID(elemItem, fileName) {

    for(var x = 0; x < elemItem.attachments.length; x++) {
        var attachment = elemItem.attachments[x];
        if(attachment.resourceName === fileName) return attachment.fileID;
    }

    return '';

}
function getFlagLabel(elemItem, row, supplier) {
    
    for(var j = elemItem.relationships.length - 1; j >= 0; j--) {
        
        var elemRelated = elemItem.relationships[j];
        
        if(elemRelated.description.indexOf(supplier + ' | ') === 0) {

            var revision = elemRelated.description.split(' | ')[1];

            row.PREVIOUS_PACKAGE = elemRelated.item;
            
            if(revision === elemItem.descriptor.descriptor) {
                return 'Repeat';
            } else {
                return 'Update';
            }
            
        }
        
    }
    
    return 'New';
    
}