// This script is part of the CHANGE MANAGEMENT app available in the App Store
// Last Modification Date : June 27, 2023

var workspaceCAPA               = "WS_CORRECTIVE_PREVENTATIVE_ACTIONS";  // Provide workpace name of CAPA to initiate followup proceses from Problem Reports. Otherwise leave blank.
var linkCAPA                    = "PROBLEM_REPORT";                      // Provide FieldID of link from CAPA to Problem Report
var fieldsCAPA                  = ["TITLE", "DESCRIPTION"];              // Provide list of fields to be copied from Problem Report to CAPA


/* Clear item details fields */
function resetField(fieldId) {
    
    if(item[fieldId] !== null) item[fieldId] = null;
    
}


/* Copy affected item selected in item details tab to list of affected items */
function setAffectedItems() {

    var affectedItem = item.AFFECTED_ITEM;

    if(affectedItem === null) return;
    if(item.workflowItems.length > 0) return;

    if(affectedItem.descriptor.dvi.working) {

        item.workflowItems.addItem({ RELATED:affectedItem });

    } else {

        for (var i in affectedItem.descriptor.revisionList) {
            var itemRevision = affectedItem.descriptor.revisionList[i];
            if (itemRevision.working) {
                var workingItem = loadItem(itemRevision.dmsID);
                item.workflowItems.addItem({ RELATED:workingItem });
                return;
            }
        }

    }

}



/* Create, update or delete defined milestone */
function setMilestone(name, event, date, offset, threshold, progress, deleteIfEmpty) {
    
    var milestoneIndex  = -1;
    var offsetSat       = (date === null) ? 2 : 2;
    var offsetSun       = (date === null) ? 1 : 1;
    var targetDate      = new Date();
    
    if(         date !== null) { targetDate = date; } else if(offset !== null) { targetDate.setDate(targetDate.getDate() + offset); } 
    if(        event === null) event         = "ENTER";
    if(    threshold === null) threshold     = 0;
    if(deleteIfEmpty === null) deleteIfEmpty = true;
    
    var day = targetDate.getDay();
    
         if(day === 0) targetDate.setDate(targetDate.getDate() + offsetSun);
    else if(day === 6) targetDate.setDate(targetDate.getDate() + offsetSat);

    for(var i = 0; i < item.milestones.length; i++) {
        if(item.milestones[i].workflowStateName == name) {
            if(item.milestones[i].milestoneType == event) {
                milestoneIndex = i;
                break;
            }
        }
    }
    
    if(deleteIfEmpty) {
        if (date === null) {
            if (offset === null) {
                if(milestoneIndex > -1) {
                    item.milestones.splice(milestoneIndex, 1);
                }
                return;
            }
        }
    }

    if(milestoneIndex == -1) {
        
		var milestone = item.addMilestone();
		    milestone.setWorkflowState(name);
		    milestone.milestoneType = event;
		    milestone.milestoneDate = targetDate;
		    milestone.warnThreshold = threshold;
            milestone.progress      = progress;
        
    } else {
        
        item.milestones[milestoneIndex].milestoneDate = targetDate;
        item.milestones[milestoneIndex].progress = progress;
        
    }
    
}


/* Save current date in defined field */
function saveDate(field) {
    
    if(item[field] === null) item[field] = new Date();
    
}


// Set responsibilities based on selected approval list template
function setCoordinator(fieldIdSource, setOwner) {
    
    if(item.CHANGE_TEMPLATE === null) item.COORDINATOR = null;
    else if(item.CHANGE_TEMPLATE[fieldIdSource] === null) item.COORDINATOR = null;
    else if(item.COORDINATOR !== item.CHANGE_TEMPLATE[fieldIdSource]) {
     
        var coordinator     = item.CHANGE_TEMPLATE[fieldIdSource];
        
        item.COORDINATOR = coordinator;
        
        if(setOwner) {
        
            var userSelected    = coordinator.split(",");
    	    var lastName        = userSelected[0].trim();
    	    var firstName       = userSelected[1].trim();
            var user            = Security.searchUsers({lastName:lastName, firstName:firstName}); 
    		            
    	    if(user !== null) {
    	        var userId = user[0].id;
    	        if(item.master.owner.id !== userId) {
    	            item.master.owner = Security.loadUser(userId);
    	        }
    	    }
     
        }
        
    }
    
}


// Sync user pick list with user list define in change template
function setUserList(fieldSource, fieldTarget) {

    if(item.CHANGE_TEMPLATE === null) {
        
        item[fieldTarget] = [];
        
    } else {

        var source = item.CHANGE_TEMPLATE[fieldSource];

        for(var i = 0; i < source.length; i++) {
        
            var sourceUser = source[i];
            var add       = true;
            
            for(var j = 0; j < item[fieldTarget].length; j++) {
                var targetUser = item[fieldTarget][j];
                if(sourceUser === targetUser) {
                    add = false;
                    continue;
                }
            }
                
            if(add) item[fieldTarget].add(sourceUser);
            
        }
        
        if(source.length !== item[fieldTarget].length) cleanupUsersList(source, fieldTarget);
    
    }
    
}
function cleanupUsersList(source, fieldTarget) {
    
    for(var i =  item[fieldTarget].length - 1; i >= 0; i--) {
            
        var targetUser = item[fieldTarget][i];
        var remove   = true;
        
        for(var j = 0; j < source.length; j++) {
            var sourceUser = source[j];
            if(targetUser === sourceUser) {
                remove = false;
                continue;
            }
        }
            
        if(remove) item[fieldTarget].remove(item[fieldTarget][i]);
        
    }
    
}


// Set remaining approvers baesed on required & additional approvers
function setRemainingApprovers(fieldRequiredApprovers, fieldAdditionalApprovers, fieldRemainingApprovers) {
    
    var approvers = [];
    
    for(var i = 0; i < item[fieldAdditionalApprovers].length; i++) {
        approvers.push(item[fieldAdditionalApprovers][i]);
    }
    
    for(var j = 0; j < item[fieldRequiredApprovers].length; j++) {
        
        var required = item[fieldRequiredApprovers][j];
        if(approvers.indexOf(required) === -1) approvers.push(required);
        
    }
    
    syncUserList(approvers, fieldRemainingApprovers);
    
}
function syncUserList(source, fieldTarget) {

    for(var i = 0; i < source.length; i++) {
        item[fieldTarget].add(source[i]);
    }
    
}


// once reaching CCB review, increase counter
function setKPIRoundCount() {
    
    var current = (item.CCB_REVIEW_ROUNDS === null) ? 0 : item.CCB_REVIEW_ROUNDS;
    item.CCB_REVIEW_ROUNDS = current + 1;
    
}


// Upon approval, save username in approved by and remove it from remaining approvers
function saveApproval(fieldApprovedBy, fieldRemainingApprovers) {
    
    var userName = getUserName();
    
    item[fieldApprovedBy].add(userName);
    item[fieldRemainingApprovers].remove(userName);
    
}


// Copy grid from selected change template to current process
function copyTaskTemplate(process) {
    
    var source = item.CHANGE_TEMPLATE;
    
    if(source === null) return;
    
    if(source !== item.COPIED_TASKS_FROM_TEMPLATE) {
        
        item.grid.clear();
        
        for(var i = 0; i < source.grid.length; i++) {
            
            var rowRef = source.grid[i];
            
            if(rowRef.PROCESS === process) {
            
                var rowNew = item.grid.addRow();
                
                var columnsRef = Object.keys(rowRef);
                var columnsNew = Object.keys(rowNew);
                
                for(var j = 0; j < columnsRef.length; j++) {
                
                    var columnRef = columnsRef[j];
            
                    for(var k = 0; k < columnsNew.length; k++) {
                        
                        var columnNew = columnsNew[k];
                
                        if(columnRef == columnNew)  rowNew[columnNew] = rowRef[columnRef];
                        
                    }
                    
                }
                
            }
        
        }
        
        item.COPIED_TASKS_FROM_TEMPLATE = source.descriptor.descriptor;
    
    }
    
}


// Check if tasks have been defined (correctly)
function validateTasksPlanning() {
    
    var result = [];
    
    if(item.grid.length === 0) {
        result.push("Change tasks must be defined in Tasks Planning tab first");
    } else {
        for(var i = 0; i < item.grid.length; i++) {
            if(item.grid[i].ASSIGNEE === null) {
                result.push("Assignee is required for all entries in tab Task Planning");
                break;
            } else if(item.grid[i].TARGET_COMPLETION_DATE === null) {
                result.push("Target Completion Date is required for all entries in tab Task Planning");
                break;
            }
            
        }        
    }
    
    return result;
}


// Check if tasks have been defined in planning but not executed yet
function validateTasksPlanningEmpty() {
    
    var result = [];
    
    if(item.grid.length > 0 ) result.push("There are entries in 'Task Planning' tab, please perform 'Initiate Tasks' transition to start these tasks");
    
    return result;
    
}


// Initiate change tasks based on grid entries
function initiateChangeTasks(fieldParent) {
    
    var countPending = 0;
    var effortPlanned = (item.TOTAL_EFFORT_PLANNED === null) ? 0 : item.TOTAL_EFFORT_PLANNED;
    
    for(var i = 0; i < item.grid.length; i++) {
        
        var row = item.grid[i];
        var elemTask = row.TASK
        
        if(elemTask === null) {
        
            elemTask = createItem("WS_CHANGE_TASKS");
            elemTask[fieldParent] = item;
            elemTask.TITLE = row.TITLE;
            elemTask.DESCRIPTION = row.DESCRIPTION;
            elemTask.TARGET_START_DATE = row.TARGET_START_DATE;
            elemTask.TARGET_COMPLETION_DATE = row.TARGET_COMPLETION_DATE;
            elemTask.TARGET_EFFORT = row.TARGET_EFFORT;
            elemTask.ASSIGNEE = row.ASSIGNEE;
            elemTask.REVIEWER = row.REVIEWER;
            elemTask.ENGINEERING_ACTIVITY = (row.ENGINEERING_ACTIVITY) ? 'Yes' : 'No';
            elemTask.PRIORITY = row.PRIORITY;

            if(row.TARGET_START_DATE === null) {
                row.TARGET_START_DATE = new Date();
                elemTask.TARGET_START_DATE = new Date();
            } else {
                elemTask.TARGET_START_DATE = row.TARGET_START_DATE;
            }
            
            row.TASK = elemTask;
            item.project.addTask(elemTask);
            
            if(row.TARGET_EFFORT !== null) effortPlanned += row.TARGET_EFFORT;
            
        } else {
            
            updateTaskField(elemTask, row, "TITLE");
            updateTaskField(elemTask, row, "DESCRIPTION");
            updateTaskField(elemTask, row, "TARGET_START_DATE");
            updateTaskField(elemTask, row, "TARGET_COMPLETION_DATE");
            updateTaskField(elemTask, row, "ASSIGNEE");
            updateTaskField(elemTask, row, "REVIEWER");
            updateTaskField(elemTask, row, "PRIORITY");

            var engineeringActivity = (row.ENGINEERING_ACTIVITY) ? 'Yes' : 'No';

            if(elemTask.ENGINEERING_ACTIVITY !== engineeringActivity) elemTask.ENGINEERING_ACTIVITY = engineeringActivity;

            if(elemTask.TARGET_EFFORT !== row.TARGET_EFFORT) {
                if(row.TARGET_EFFORT !== null) effortPlanned += row.TARGET_EFFORT;
                if(elemTask.TARGET_EFFORT !== null) effortPlanned -= elemTask.TARGET_EFFORT;
                elemTask.TARGET_EFFORT = row.TARGET_EFFORT;
            }
            
        }
        
        
        elemTask.COMMENT = item.workflowActions[0].comments;
        countPending++;
        
    }
    
    item.PENDING_TASKS_COUNT = countPending;
    item.TOTAL_TASKS_COUNT = item.project.children.length;
    
    if(item.TOTAL_EFFORT_PLANNED !== effortPlanned) item.TOTAL_EFFORT_PLANNED = effortPlanned;
    
}
function updateTaskField(elemTask, row, fieldID) {
    
    if(elemTask[fieldID] !== row[fieldID]) elemTask[fieldID] = row[fieldID];
    
}


// Transition to next status if all approvals are complete
function autoTransition(fieldIdUsers, transition, comment) {
    if(item[fieldIdUsers].length === 1) {
        item.performWorkflowTransitionByCustomID(transition, comment);
    }
}


// Transition preceding process upon completion
function transitionPrecedingProcess(fieldId, status, transition, comments) {
    
    var process = item[fieldId];
    
    if(process === null) return;
    
    if(process.descriptor.workflowState !== status) return;
    
    process.performWorkflowTransitionByCustomID(transition, comments);
    
}
function transitionPrecedingProcesses(fieldId, status, transition, comments) {
    
    var processes = item[fieldId];
    
    for(var i = 0; i < processes.length; i++) {
    
        var process = processes[i];
        if(process.descriptor.workflowState === status) {
            process.performWorkflowTransitionByCustomID(transition, comments);
        }
        
    }
    
}


/* ----------------------------------------------------------------------------------------------------
    Create follow up process, link each other and copy attributes
    - workspace          : ID of workspace in which new process needs to be created
    - linkBack           : ID of field in new workspaces to link back to current item
    - linkNext           : ID of field in current workspaces to link to new item
    - copyMatchingFields : toggle to copy values of fields having same field id (true/false)
    - copyManagedItems   : toggle to copy list of managed items to new object (true/false)
    ---------------------------------------------------------------------------------------------------- */
function createFollowUpProcess(workspace, linkBack, linkNext, copyFields, copyManagedItems) {
    
    var followUp = (linkNext !== null) ? item[linkNext] : null;
    
    if(followUp !== null) {
        
        followUp[linkBack].add(item);
        
    } else {
    
        followUp = createItem(workspace);
        
        if(linkBack !== null) followUp[linkBack] = item;
        if(linkNext !== null) item[linkNext] = followUp;
        
        var copyFieldIDs = copyFields.split(",");
        
        for(var i = 0; i < copyFieldIDs.length; i++) {
            followUp[copyFieldIDs[i]] = item[copyFieldIDs[i]];
        }
        
        if(copyManagedItems !== null) {
            if(copyManagedItems) {
                for(var j = 0; j < item.workflowItems.length; j++) {
                    var workflowItem = item.workflowItems[j].item;
                    followUp.workflowItems.addItem({RELATED:workflowItem});
                }
            }
        }
    
    }
    
    return followUp;
    
}


/* ----------------------------------------------------------------------------------------------------
    Remove given existing milestones
   ---------------------------------------------------------------------------------------------------- */
function removeMilestones(name) {
    
    for(var i = item.milestones.length - 1; i >= 0; i--) {
        if(item.milestones[i].workflowStateName === name) {
            item.milestones.splice(i, 1);
        }
    }
}


/* ----------------------------------------------------------------------------------------------------
    Remove all existing milestones
   ---------------------------------------------------------------------------------------------------- */
function clearMilestones() {
    
    for(var i = 0; i < item.milestones.length; i++) {
        item.milestones.splice(i, 1);
    } 
    
}


/* ----------------------------------------------------------------------------------------------------
    Capture workflow comment in paragraph field to save feedback in thread
    - field : Paragraph field to capture discussion thread
    - append : Append transition details (true) or save last transistion details only (false)
    ---------------------------------------------------------------------------------------------------- */
function appendToThread(field, append) {
    
    if(item.workflowActions[0].transition.fromState === null) return;
    if(item.workflowActions[0].comments === null) return;
    if(item.workflowActions[0].comments === "") return;
    
    var comment = item.workflowActions[0].comments;
    var date    = new Date();
    var thread  = "&lt;p&gt;";
    
    thread += "&lt;span style='font-size:0.8em;font-style:italic;color:#ffa600;white-space:nowrap;'&gt;";
    thread += item.workflowActions[0].transition.fromState.shortName + " &gt; " + item.workflowActions[0].transition.toState.shortName;
    thread += "&lt;/span&gt;";
    thread += "&lt;br&gt;";
   
    if(comment !== null) {
        if(comment !== "") {
            thread += comment + "&lt;br&gt;";
        }
    }
    
    thread += "&lt;span style='font-size:0.8em;font-style:italic;color:#999;white-space:nowrap;'&gt;";
    thread += getUserName() + " (" + date.toDateString() + ")";
    thread += "&lt;/span&gt;";
    thread += "&lt;/p&gt;";
            
    if(append) {
        if(item[field] !== null) thread += item[field];
    }
            
    item[field] = thread;
    
}


/* Get current user's name */
function getUserName() {
    var usr = Security.loadUser(userID);
    return usr.lastName + ", " + usr.firstName;
}