// This script is part of the NEW PRODUCT INTRODUCTION app available in the App Store
// Last modification date: Nov 17, 2022

var phases = [
    "Specification",
    "Concept",
    "Development",
    "Validation",
    "Production"
];

var wsProjectPhases = "WS_PROJECT_PHASES";
var wsProjectTasks  = "WS_PROJECT_TASKS";

var statePhaseInWork   = 'In Work';
var stateTaskCompleted = 'Completed';


// Get current user's name
function getUserName() {
    var usr = Security.loadUser(userID);
    return usr.lastName + ", " + usr.firstName;
}


// Calculate durations of project templates on demand and upon release
function updateProjectTemplate() {

    var projectTemplateTotal = 0;
    
    for(var i = 1; i <= phases.length; i++) {
        item["DURATION_PHASE_" + i] = getDuration(phases[i - 1]);
        projectTemplateTotal += item["DURATION_PHASE_" + i];
        projectTemplateTotal += addReviewDuration(i);
    }
    
    item.DURATION = projectTemplateTotal;
    
}
function getDuration(phaseName) {

    var tasks       = [];
    var max         = 0;
    var duration    = 0;

    for(var x = 0 ; x < item.grid.length; x++) {
    
        var row = item.grid[x];

        if(row.PHASE === phaseName) {
    
            tasks.push({
                "id"        : row.ID,
                "pre"       : row.PREDECESSOR,
                "duration"  : row.DURATION,
                "offset"    : 0,
                "row"       : x
            });
    
            if(row.PREDECESSOR !== null) {
                if(row.PREDECESSOR > max) {
                    max = row.PREDECESSOR;
                }            
            }
            
            if(row.DURATION > duration) duration = row.DURATION;
            
        }
    
    }
    
    for(var i = 0; i <= max; i++) {
        
        var add = 0;
        
        for(var j = 0; j < tasks.length; j++) {
            if(i === tasks[j].id) {
                add = tasks[j].duration;
                break;
            }
        }
        
        if(add > 0) {
            for(var k = 0; k < tasks.length; k++) {
                if(i === tasks[k].pre) {
                    tasks[k].duration += add;
                    tasks[k].offset += add;
                    if(tasks[k].duration > duration) duration = tasks[k].duration;
                }
            }
        }
        
    }
    
    return duration;

}
function addReviewDuration(index) {
    
    if(item["DURATION_REVIEW_" + index] === null) return 0;
    else return item["DURATION_REVIEW_" + index];
    
}


// Skip non working days
function getNextWorkingDay(elemStart, offset) {
    
    var elemDate = new Date(elemStart.getTime());
        elemDate.setDate(elemDate.getDate() + offset);
    
         if(elemDate.getDay() === 6) elemDate.setDate(elemDate.getDate() + 2);
    else if(elemDate.getDay() === 0) elemDate.setDate(elemDate.getDate() + 1);
    
    return new Date(elemDate.getTime());
    
}


// Determine end date based on number of working days
function getWorkingDaysEnd(dateStart, duration) {
    
    if(duration === 0) return new Date(dateStart.getTime());
    
    duration -= 1;

    var endDate = new Date(dateStart.getTime());
    var dow     = dateStart.getDay();
    
    if (dow == 0) duration++;

    if (dow + duration >= 6) {
        var remainingWorkDays = duration - (5 - dow);
        duration += 2;
        if (remainingWorkDays > 5) {
            duration += 2 * Math.floor(remainingWorkDays / 5);
            if (remainingWorkDays % 5 == 0)
                duration -= 2;
        }
    }
    
    endDate.setDate(endDate.getDate() + duration);
    
    return new Date(endDate.getTime());
    
}


// Set entry in Gantt for Gates and Phases of projects
function setProjectPhase(elemProject, title, start, end, progress) {
    
    var index   = -1;
    var project = elemProject.project;
    
    for(var i = 0; i < project.children.length; i++) {
        if(title === project.children[i].title) {
            index = i;
            break;
        }
    }
    
    if(start !== null) {
        
        if(end < start) {
            
            if(index >= 0) {
                project.children[index].remove();
            }
            
        } else {

            if(index >=0) {
                var task = project.children[index];
                    task.progress = progress;
                    task.start_date = start;
                    task.end_date = end;
            } else {
                item.project.addTask(title, start, end, progress);
            }
            
        }  
        
    } else if(index >= 0) {
        
        project.children[index].remove();
        
    }
    
}
function updateProjectPhase(elemProject, title, progress) {
    
    var project = elemProject.project;
    
    for(var i = 0; i < project.children.length; i++) {
        if(title === project.children[i].title) {
            project.children[i].progress = progress;
            return;
        }
    }
    
}


// Update schedule of upcoming phases
function updatePhaseTasks(elemProject, elemPhase, dateBase, indexPhase) {
    
    var tasks       = [];
    var min         = 1000000;
    var max         = -1;
    var phaseEnd    = dateBase;
    var phaseStart  = dateBase;
    var phaseEffort = 0;
    var buffer      = elemProject["DURATION_REVIEW_" + indexPhase];
    
    elemPhase.project.clear();
    
    for(var x = 0 ; x < elemPhase.grid.length; x++) {
    
        var row = elemPhase.grid[x];
    
        var task = {
            "id"        : row.ID,
            "pre"       : row.PREDECESSOR,
            "duration"  : row.PLANNED_DURATION,
            "title"     : row.TITLE,
            "offset"    : 0,
            "row"       : x,
            "fixStart"  : row.FIXED_START_DATE,
            "fixEnd"    : row.FIXED_COMPLETION_DATE
        };

        if(row.PREDECESSOR !== null) {
            if(row.PREDECESSOR > max) max = row.PREDECESSOR;
            if(row.PREDECESSOR < min) min = row.PREDECESSOR;
        } else {
            var endDate = getWorkingDaysEnd(phaseStart, task.duration);
            task.start = new Date(phaseStart.getTime());
            task.end = new Date(endDate.getTime());
        }
        
        if(row.FIXED_START_DATE !== null) task.start = row.FIXED_START_DATE;
        if(row.FIXED_COMPLETION_DATE !== null) task.end = row.FIXED_COMPLETION_DATE;

        phaseEffort += (row.PLANNED_EFFORT === null) ? 0 : row.PLANNED_EFFORT;

        tasks.push(task);
    
    }
    
    for(var i = min; i <= max; i++) {
        
        var predEnd;
        
        for(var j = 0; j < tasks.length; j++) {
            if(i === tasks[j].id) {
                predEnd = tasks[j].end;
                break;
            }
        }
        
        var newStart = getNextWorkingDay(predEnd, 1);
        
        
        if(predEnd !== null) {
            for(var k = 0; k < tasks.length; k++) {
                if(i === tasks[k].pre) {
                    var newEnd = getWorkingDaysEnd(newStart, tasks[k].duration);
                    tasks[k].start = newStart;
                    tasks[k].end = newEnd;
                }
            }
        }
        
    }
    
    for(var l = 0; l < tasks.length; l++) {

        var task    = tasks[l];
        
        elemPhase.grid[l].PLANNED_START_DATE = task.start;
        elemPhase.grid[l].PLANNED_COMPLETION_DATE = task.end;
        
        elemPhase.project.addTask(task.title, task.start, task.end);
  
        if(task.end.getTime() > phaseEnd.getTime()) phaseEnd = new Date(task.end.getTime());
        
    }
    
    if(buffer === null) buffer = 1;
    
    if(elemProject.STEP < 1) {
        
        elemPhase.PLANNED_START_DATE                          = phaseStart;
        elemPhase.PLANNED_COMPLETION_DATE                     = phaseEnd;
        elemPhase.PLANNED_EFFORT                              = phaseEffort;
        elemProject["PLANNED_COMPLETION_PHASE_" + indexPhase] = phaseEnd;
        elemProject["PLANNED_EFFORT_PHASE_"     + indexPhase] = phaseEffort;
        
    } else {
    
        elemPhase.EXPECTED_START_DATE                           = phaseStart;
        elemPhase.EXPECTED_COMPLETION_DATE                      = phaseEnd;
        elemPhase.EXPECTED_EFFORT                               = phaseEffort;
        elemProject["EXPECTED_COMPLETION_PHASE_" + indexPhase]  = phaseEnd;
        elemProject["EXPECTED_EFFORT_PHASE_"     + indexPhase]  = phaseEffort;
        
    }
    
    var buffer = elemProject["DURATION_REVIEW_" + indexPhase];
    
    if((buffer === 0) || (buffer === null)) {
        
        if(elemProject.STEP < 1) {
            elemProject["PLANNED_COMPLETION_GATE_" + indexPhase] = phaseEnd;
        } else {
            elemProject["EXPECTED_COMPLETION_GATE_" + indexPhase] = phaseEnd;
        }
        
        return getNextWorkingDay(phaseEnd, 1);
      
    
    } else {
        
        var planEnd     = elemProject["PLANNED_COMPLETION_GATE_" + indexPhase];
        var reviewStart = getNextWorkingDay(phaseEnd, 1);
        var reviewEnd;

        if(planEnd.getTime() <= phaseEnd.getTime()) {
            reviewEnd = getNextWorkingDay(phaseEnd, 1);
        } else if(phaseEnd.getTime() > elemProject["PLANNED_COMPLETION_PHASE_" + indexPhase].getTime()) {
            reviewEnd = new Date(planEnd.getTime());
        } else {
            reviewEnd = getWorkingDaysEnd(reviewStart, buffer);
        }

        setProjectPhase(elemProject, "G" + indexPhase, reviewStart, reviewEnd, 0);

        if(elemProject.STEP < 1) {
            elemProject["PLANNED_COMPLETION_GATE_" + indexPhase] = reviewEnd;
        } else {
            elemProject["EXPECTED_COMPLETION_GATE_" + indexPhase] = reviewEnd;
        }

        return getNextWorkingDay(reviewEnd, 1);
        
    }

}



// Sync task list of project's grid with related tasks
function updateProjectTasks() {
 
    var indexPhase      = phases.indexOf(item.descriptor.workflowState) + 1;
    var effortExpected  = item["ACTUAL_EFFORT_PHASE_" + indexPhase];
    var durationTotal   = 0;
    var progressTotal   = 0;
    
    if(indexPhase === -1) return;
    
    if(effortExpected === null) effortExpected = 0;
    
    for(var i = 0; i < item.grid.length; i++) {
     
        var row      = item.grid[i];
        var elemTask = row.TASK;
        
        if(elemTask === null) {
            
            var effort = (row.PLANNED_EFFORT === null) ? 0 : row.PLANNED_EFFORT;
            elemTask   = createProjectTask(row, item, item.CURRENT_PHASE, effort);
            row.TASK   = elemTask;
            row.PHASE  = item.descriptor.workflowState;
            
            durationTotal += row.PLANNED_DURATION;
            effortExpected += effort;
            
        } else {
            
            updateTaskProperty(row, "TITLE");
            updateTaskProperty(row, "DESCRIPTION");
            updateTaskProperty(row, "PRIORITY");
            updateTaskProperty(row, "PLANNED_START_DATE");
            updateTaskProperty(row, "PLANNED_COMPLETION_DATE");
            updateTaskProperty(row, "PLANNED_EFFORT");
            updateTaskProperty(row, "TEAM");
            updateTaskProperty(row, "TEAM_LEADER");
            updateTaskProperty(row, "ASSIGNEE");
            updateTaskProperty(row, "REVIEWER");
            
            elemTask.PLAN_TEAM      = row.TEAM;
            elemTask.PLAN_ASSIGNEE  = row.ASSIGNEE;
            elemTask.PLAN_REVIEWER  = row.REVIEWER;
            
            if(row.PLANNED_EFFORT !== null) effortExpected += row.PLANNED_EFFORT;
            
        }
        
    }
    
    item["EXPECTED_EFFORT_PHASE_" + indexPhase] = effortExpected;
    
    if(item.CURRENT_PHASE === null) return;
    
    var tasks = item.CURRENT_PHASE.project.children;
    
    for(var i = tasks.length - 1; i >= 0; i--) {
        
        var durationTask = Number(tasks[i].duration.split("d")[0]) + 1;
        var progressTask = progressTask = tasks[i].progress * durationTask;

        progressTotal += progressTask
        durationTotal += durationTask;
    }
    
    var progress = Math.round((progressTotal / durationTotal), 0);
    
    if(progressTotal === 0) { if(durationTotal === 0) {
        progress = 0;
    }}
    
    item.DESCRIPTION = progress;
    
    item.CURRENT_PHASE.PERCENT_COMPLETE = progress;
    item.CURRENT_PHASE_PERCENT_COMPLETE = progress;
    
}
function updateTaskProperty(row, fieldId) {
 
    if(row[fieldId] !== row.TASK[fieldId]) row.TASK[fieldId] = row[fieldId];
    
}



// Create new Project Task records
function createProjectTask(row, elemProject, elemPhase, effort) {
    
    var elemTask = createItem(wsProjectTasks);
    
    setTaskProperty(elemTask, row, "TITLE");
    setTaskProperty(elemTask, row, "DESCRIPTION");
    setTaskProperty(elemTask, row, "PRIORITY");
    setTaskProperty(elemTask, row, "PLANNED_START_DATE");
    setTaskProperty(elemTask, row, "PLANNED_COMPLETION_DATE");
    setTaskProperty(elemTask, row, "TEAM");
    setTaskProperty(elemTask, row, "TEAM_LEADER");
    setTaskProperty(elemTask, row, "REVIEWER");
    
    elemTask.PLANNED_EFFORT = effort;
    elemTask.PHASE          = elemPhase;
    elemTask.PROJECT        = elemProject;
    elemTask.PROJECT_NUMBER = elemProject.IDPRJ;
    elemTask.PLAN_TEAM      = row.TEAM;
    elemTask.PLAN_ASSIGNEE  = row.ASSIGNEE;
    elemTask.PLAN_REVIEWER  = row.REVIEWER;
    
    if(row.ASSIGNEE !== null) {
        elemTask.ASSIGNEE = row.ASSIGNEE;
    } else if(row.TEAM !== null) {
        var member = getProjectMember(row.TEAM);
        if(member !== null) {
            elemTask.ASSIGNEE = member;
        }
    }
    
    return elemTask;
    
}
function setTaskProperty(elemTask, row, field) {
    
    if(row[field] !== null) {
        elemTask[field] = row[field];
    }
    
}
function getProjectMember(teamName) {
    
    for(var i = 1; i < 6; i++) {
        if(item["TEAM_" + i] !== null) {
            if(item["TEAM_" + i].NAME === teamName) {
                return item["MEMBER_" + i];
            }
        }
    }
    
    return null;
    
}



// When changing current project tasks, reschedule all future phases
function updateProjectScheduleFromPhase() {
    
    if(item.CURRENT_PHASE === null) return;
    
    var currentPhase    = item.descriptor.workflowState;
    var indexPhase      = phases.indexOf(currentPhase) + 1;
    var phaseEnd        = null;
    var phaseEffort     = item.CURRENT_PHASE.ACTUAL_EFFORT;
    
    if(phaseEffort === null) phaseEffort = 0;
    
    for(var i = 0; i < item.grid.length; i++) {
        
        if(item.grid[i].PHASE === currentPhase) {
        
            var targetDate = item.grid[i].PLANNED_COMPLETION_DATE;
        
            if(targetDate !== null) {
                if(phaseEnd === null) {
                    phaseEnd = new Date(targetDate.getTime());
                  } else if(targetDate.getTime() > phaseEnd.getTime()) {
                      phaseEnd = new Date(targetDate.getTime());
                  }
            }
            
            phaseEffort += item.grid[i].PLANNED_EFFORT;
            
        }
        
    }
    
    item["EXPECTED_COMPLETION_PHASE_" + indexPhase] = phaseEnd;
    item.CURRENT_PHASE.ACTUAL_EFFORT = phaseEffort;
   
    var buffer      = item["DURATION_REVIEW_" + indexPhase];
    var planEnd     = item["PLANNED_COMPLETION_GATE_" + indexPhase];
    var reviewStart = getNextWorkingDay(phaseEnd, 1);
    var reviewEnd;
    
    if(planEnd.getTime() <= phaseEnd.getTime()) {
        reviewEnd = getNextWorkingDay(phaseEnd, 1);
    } else if(phaseEnd.getTime() > item["PLANNED_COMPLETION_PHASE_" + indexPhase].getTime()) {
        reviewEnd = new Date(planEnd.getTime());
    } else {
        reviewEnd = getWorkingDaysEnd(reviewStart, buffer);
    }
    
    setProjectPhase(item, "G" + indexPhase, reviewStart, reviewEnd, 0);
    
    item["EXPECTED_COMPLETION_GATE_" + indexPhase] = reviewEnd;
    
    var dateBase = getNextWorkingDay(reviewEnd, 1);
    
    for(var i = indexPhase  + 1; i <= phases.length; i++) {
        dateBase = updatePhaseTasks(item, item["PHASE_" + i], dateBase, i);
    }
    
}