// This script is part of the PRODUCT PORTFOLIO MANAGEMENT app available in the App Store
// Last modification date: May 14, 2021


var weighting    = [30, 25, 25, 10, 10];
var milestones   = ["Product Development", "Product Launch", "Sales", "Maintenance", "End of Life"];
var statesInWork = ["Proposal", "Product Selection", "Product Development", "Product Launch"];


// Get current user's name
function getUserName() {
    var usr = Security.loadUser(userID);
    return usr.lastName + ", " + usr.firstName;
}


// Set, update and delete milestones
function setMilestone(name, event, date, offset, threshold, progress, deleteIfEmpty, skipWeekends) {
    
    var milestoneIndex  = -1;
    var offsetSat       = (date === null) ? 2 : 2;
    var offsetSun       = (date === null) ? 1 : 1;
    var targetDate      = new Date();
    
    if(         date !== null) { targetDate = date; } else if(offset !== null) { targetDate.setDate(targetDate.getDate() + offset); } 
    if(        event === null) event         = "ENTER";
    if(    threshold === null) threshold     = 0;
    if(deleteIfEmpty === null) deleteIfEmpty = true;
    
    var day = targetDate.getDay();
    
    
    if(skipWeekends) {
             if(day === 0) targetDate.setDate(targetDate.getDate() + offsetSun);
        else if(day === 6) targetDate.setDate(targetDate.getDate() + offsetSat);
    }
        
    for(var i = 0; i < item.milestones.length; i++) {
        if(item.milestones[i].workflowStateName == name) {
            if(item.milestones[i].milestoneType == event) {
                milestoneIndex = i;
                break;
            }
        }
    }
    
    if(deleteIfEmpty) {
        if (date === null) {
            if (offset === null) {
                if(milestoneIndex > -1) {
                    item.milestones.splice(milestoneIndex, 1);
                }
                return;
            }
        }
    }

    if(milestoneIndex == -1) {
        
		var milestone = item.addMilestone();
		    milestone.setWorkflowState(name);
		    milestone.milestoneType = event;
		    milestone.milestoneDate = targetDate;
		    milestone.warnThreshold = 0;
            milestone.progress      = progress;
        
    } else {
        
        item.milestones[milestoneIndex].milestoneDate = targetDate;
        item.milestones[milestoneIndex].progress = progress;
        
    }
    
}


/* ----------------------------------------------------------------------------------------------------
    Add current item to project management tab of item selected in pick list field
    - fieldNew  : Pick list field in which users can select the parent (where the current item will be added)
    - fieldOld  : Pick list field to store previously selected parent (will be set by script automatically)
   ---------------------------------------------------------------------------------------------------- */
function updateRelatedProject(fieldNew, fieldOld) {
    
    var elemNew = item[fieldNew];
    var elemOld = item[fieldOld];
    
    
   /* if(fieldOld === null) {
        elemNew.project.addTask(item);  
        
    } else */
    
    if(elemNew !== elemOld) {

        if(elemNew !== null) elemNew.project.addTask(item); 
        
        if(elemOld !== null) {
        
            var elemTasks = elemOld.project.children;
            var itemId = item.master.dmsID;
            
            for(var i = elemTasks.length - 1; i >= 0; i--) {
                
                var elemTask = elemTasks[i];
                
                if(elemTask !== null) {
                    
                    if(elemTask.item !== null) {
                
                        var elemTaskId = elemTask.item.master.dmsID;
                        
                        if(elemTaskId == itemId) elemTask.remove(); 
                        
                    }
                    
                }
                
            }
        
        }
        
        item[fieldOld] = item[fieldNew];
        
    
    }

}
function updateRelatedProjects(fieldNew, fieldOld) {
    
    var elementsNew = item[fieldNew];
    var elementsOld = item[fieldOld];
    
    var idsNew = getObjectIdsOfPickList(elementsNew);
    var idsOld = getObjectIdsOfPickList(elementsOld);
    var itemId = item.master.dmsID;
    
    if(idsNew !== idsOld) {
        
        for(var i = 0; i < elementsNew.length; i++) {
            
            var elementNew   = elementsNew[i];
            var elementIdNew = elementNew.master.dmsID;
            
            if(idsOld.indexOf(elementIdNew) == -1) { 
                elementNew.project.addTask(item);  
                elementsOld.add(elementNew);
            }
        
        }
        
        for(var j = elementsOld.length - 1; j >= 0; j--) {
            
            var elementOld = elementsOld[j];
            var elementIdOld = elementOld.master.dmsID;
            
            if(idsNew.indexOf(elementIdOld) == -1) { 
                
                var elemTasks = elementOld.project.children;
        
                for(var k = elemTasks.length - 1; k >= 0; k--) {
                    
                    var elemTask = elemTasks[k];
                    var elemTaskId = elemTask.item.master.dmsID;
                    
                    if(elemTaskId == itemId) elemTask.remove(); 
                    
                }
                
                elementsOld.remove(elementOld);
                
            }
        
        }

    }
    
}
function getObjectIdsOfPickList(elements) {
    
    var result = "";
    
    for(var i = 0; i < elements.length; i++) { result += elements[i].master.dmsID + ";"; }
    
    return result;
    
}
function removeFromProject(elemParent) {

    if(elemParent === null) return;
    
    var elemTasks = elemParent.project.children;
    var itemId = item.master.dmsID;

    for(var i = elemTasks.length - 1; i >= 0; i--) {
        var elemTask = elemTasks[i];
        if(elemTask !== null) {
            if(elemTask.item !== null) {
                var elemTaskId = elemTask.item.master.dmsID;
                if(elemTaskId == itemId) elemTask.remove(); 
            }
        }
    }
    
    
}



/* ----------------------------------------------------------------------------------------------------
    Add current object with details to selected parent's grid
    - fieldNew   : Pick list field that defines the parent whose grid needs to be updated
    - fieldOld   : Pick list field to store that last selected parent (to recognize changes)
    - columnDate : Column of grid to store date of addition to grid (optional)
    ---------------------------------------------------------------------------------------------------- */
function updateRelatedGrids(fieldNew, fieldOld, columnDate) {
    
    var elementsNew = item[fieldNew];
    var elementsOld = item[fieldOld];
    
    var idsNew = getObjectIdsOfPickList(elementsNew);
    var idsOld = getObjectIdsOfPickList(elementsOld);
    var itemId = item.master.dmsID;
    
    if(idsNew !== idsOld) {
        
        for(var i = 0; i < elementsNew.length; i++) {
            
            var elementNew   = elementsNew[i];
            var elementIdNew = elementNew.master.dmsID;
            
            if(idsOld.indexOf(elementIdNew) == -1) { 
                
                addToGrid(elementNew, columnDate);
                elementsOld.add(elementNew);
                
            } else {
                
                updateExistingGrid(elementNew);
                
            }
        
        }
        
        for(var j = elementsOld.length - 1; j >= 0; j--) {
            
            var elementOld = elementsOld[j];
            var elementIdOld = elementOld.master.dmsID;
            
            if(idsNew.indexOf(elementIdOld) == -1) { 
                
                removeFromGrid(elementOld);
                elementsOld.remove(elementOld);
                
            }
        
        }

    } else {
        
        for(var k = 0; k < elementsNew.length; k++) { updateExistingGrid(elementsNew[k]); }
        
    }
    
}
function updateRelatedGrid(fieldNew, fieldOld, columnLink) {
    
    var valueNew = item[fieldNew];
    var valueOld = item[fieldOld];
    
    if(valueNew !== valueOld) {
    
        addToGrid(valueNew, columnLink);
        removeFromGrid(valueOld, columnLink);
    
    } else {
        
        updateExistingGrid(valueNew, columnLink);
        
    }

}
function updateExistingGrid(parent, columnLink) {
    
    if(parent !== null) {
    
        var bAdd    = true;
        var grid    = parent.grid;
        
        for(var i = 0; i < grid.length; i++) {
            var row = grid[i];
            if(row[columnLink] == item) {
                bAdd = false;
                syncGridRow(row, item);
            }
        }
        
        if(bAdd) addToGrid(parent);
    
    }
    
}
function addToGrid(parent, columnLink) {

    if(parent !== null) {

        var grid = parent.grid;
        var row  = grid.addRow();
        
        syncGridRow(row, item, columnLink);
        
    }
    
}
function syncGridRow(row, item, columnLink) {
    
    var columns = Object.keys(row);
    var fields  = Object.keys(item);
    
    for(var i = 0; i < columns.length; i++) {
        
        var column = columns[i];
    
        if(column == columnLink) { row[columnLink] = item; }
        else if(column == "WORKFLOW_STATUS") { row.WORKFLOW_STATUS = item.descriptor.workflowState; }
        else {
            
            for(var j = 0; j < fields.length; j++) {
        
                var field = fields[j];
                if(field == column)  row[column] = item[field];
                
            }
            
        }
    
    }
    
}
function removeFromGrid(parent, columnLink) {
    
   if(parent !== null) {
    
        var grid = parent.grid;
        
        for(var i = grid.length - 1; i >= 0; i--) {
            
            var row = grid[i];
            if(row[columnLink] == item) row.remove();
            
        }
    
   }
    
}